# -*- coding: utf-8 -*-
"""File generated according to PHoleM54/gen_list.json
WARNING! All changes made in this file will be lost!
"""
from pyleecan.GUI.Dialog.DMachineSetup.SMHoleMag.PHoleM54.Ui_PHoleM54 import Ui_PHoleM54


class Gen_PHoleM54(Ui_PHoleM54):
    def setupUi(self, PHoleM54):
        """Abstract class to update the widget according to the csv doc"""
        Ui_PHoleM54.setupUi(self, PHoleM54)
        # Setup of in_R1
        txt = self.tr(u"""Hole radius""")
        self.in_R1.setWhatsThis(txt)
        self.in_R1.setToolTip(txt)

        # Setup of lf_R1
        self.lf_R1.validator().setBottom(0)
        txt = self.tr(u"""Hole radius""")
        self.lf_R1.setWhatsThis(txt)
        self.lf_R1.setToolTip(txt)

        # Setup of in_W0
        txt = self.tr(u"""Hole angular width""")
        self.in_W0.setWhatsThis(txt)
        self.in_W0.setToolTip(txt)

        # Setup of lf_W0
        self.lf_W0.validator().setBottom(0)
        txt = self.tr(u"""Hole angular width""")
        self.lf_W0.setWhatsThis(txt)
        self.lf_W0.setToolTip(txt)

        # Setup of in_H0
        txt = self.tr(u"""Hole depth""")
        self.in_H0.setWhatsThis(txt)
        self.in_H0.setToolTip(txt)

        # Setup of lf_H0
        self.lf_H0.validator().setBottom(0)
        txt = self.tr(u"""Hole depth""")
        self.lf_H0.setWhatsThis(txt)
        self.lf_H0.setToolTip(txt)

        # Setup of in_H1
        txt = self.tr(u"""Hole width""")
        self.in_H1.setWhatsThis(txt)
        self.in_H1.setToolTip(txt)

        # Setup of lf_H1
        self.lf_H1.validator().setBottom(0)
        txt = self.tr(u"""Hole width""")
        self.lf_H1.setWhatsThis(txt)
        self.lf_H1.setToolTip(txt)



from ....Functions.Geometry.merge_notch_list import merge_notch_list


def get_notch_list(self, sym=1):
    """Returns an ordered description of the notches

    Parameters
    ----------
    self : Lamination
        A Lamination object
    sym: int
        Number of symmetry

    Returns
    -------
    notch_list : list
        list of dictionary with key: "begin_angle", "end_angle", "obj"
    """

    if self.notch is None or len(self.notch) == 0:  # No notch
        return list()
    else:
        notch_list = self.notch[0].get_notch_list(sym=sym)
        # If more than one notch list, we need to merge in order the description
        # (check if notches are coliding)
        for ii in range(len(self.notch) - 1):
            notch_list = merge_notch_list(
                notch_list, self.notch[ii + 1].get_notch_list(sym=sym)
            )
        return notch_list



# -*- coding: utf-8 -*-

from ....Methods import ParentMissingError


def is_outwards(self):
    """Return if the slot is outwards (on an external lamination) or inwards
    (on an internal lamination)

    Parameters
    ----------
    self : Slot
        A Slot object

    Returns
    -------
    is_outwards: bool
        True if the Lamination is not internal and false if not
    """
    if self.parent is not None:
        return self.parent.is_outwards()
    else:
        raise ParentMissingError(
            "Error: The slot is not inside a Lamination or a Notch"
        )



# -*- coding: utf-8 -*-


def comp_surface_magnets(self):
    """Compute the surface of all the magnet(s)

    Parameters
    ----------
    self : HoleMag
        A HoleMag object

    Returns
    -------
    Smag: float
        Surface of all the Magnet(s) [m**2]
    """

    mag_list = self.get_magnet_list()
    Smag = 0
    for ii in range(len(mag_list)):
        Smag += self.comp_surface_magnet_id(ii)
    return Smag



# -*- coding: utf-8 -*-

from numpy import exp

from ....Classes.Segment import Segment
from ....Classes.SurfLine import SurfLine


def build_geometry(self, alpha=0, delta=0, is_simplified=False):
    """Compute the curve (Line) needed to plot the Slot.
    The ending point of a curve is the starting point of the next curve in
    the list

    Parameters
    ----------
    self : HoleM52
        A HoleM52 object
    alpha : float
        Angle to rotate the slot (Default value = 0) [rad]
    delta : complex
        Complex to translate the slot (Default value = 0)
    is_simplified : bool
       True to avoid line superposition

    Returns
    -------
    surf_list: list
        List of SurfLine needed to draw the HoleM51

    """
    if self.get_is_stator():  # check if the slot is on the stator
        st = "_Stator"
    else:
        st = "_Rotor"

    point_dict = self._comp_point_coordinate()
    Z1 = point_dict["Z1"]
    Z2 = point_dict["Z2"]
    Z3 = point_dict["Z3"]
    Z4 = point_dict["Z4"]
    Z6 = point_dict["Z6"]
    Z7 = point_dict["Z7"]
    Z8 = point_dict["Z8"]
    Z9 = point_dict["Z9"]
    Z10 = point_dict["Z10"]
    Z11 = point_dict["Z11"]

    # Creation of the air curve
    curve_list = list()
    curve_list.append(Segment(Z1, Z2))
    curve_list.append(Segment(Z2, Z3))
    curve_list.append(Segment(Z3, Z11))
    curve_list.append(Segment(Z11, Z1))
    point_ref = (Z1 + Z2 + Z3 + Z11) / 4
    S1 = SurfLine(line_list=curve_list, label="Hole" + st, point_ref=point_ref)

    # Creation of the magnet curve
    curve_list = list()
    if is_simplified:
        curve_list.append(Segment(Z3, Z11))
        curve_list.append(Segment(Z7, Z10))
    else:
        curve_list.append(Segment(Z4, Z11))
        curve_list.append(Segment(Z11, Z10))
        curve_list.append(Segment(Z10, Z6))
        curve_list.append(Segment(Z6, Z4))
    point_ref = (Z11 + Z4 + Z6 + Z10) / 4
    # Defining type of magnetization of the magnet
    if self.magnet_0:
        if self.magnet_0.type_magnetization == 0:
            type_mag = "_Radial"
        else:
            type_mag = "_Parallel"
    else:
        type_mag = "None"
    magnet_label = "HoleMagnet" + st + type_mag + "_N_R0_T0_S0"
    S2 = SurfLine(line_list=curve_list, label=magnet_label, point_ref=point_ref)

    # Creation of the second air curve
    curve_list = list()
    curve_list.append(Segment(Z7, Z8))
    curve_list.append(Segment(Z8, Z9))
    curve_list.append(Segment(Z9, Z10))
    curve_list.append(Segment(Z10, Z7))
    point_ref = (Z7 + Z8 + Z9 + Z10) / 4
    S3 = SurfLine(line_list=curve_list, label="Hole" + st, point_ref=point_ref)

    # Area with no magnet (S1 + S2 + S3)
    curve_list = list()
    curve_list.append(Segment(Z1, Z2))
    curve_list.append(Segment(Z2, Z3))
    if self.H2 > 0:
        curve_list.append(Segment(Z3, Z4))
    curve_list.append(Segment(Z4, Z6))
    if self.H2 > 0:
        curve_list.append(Segment(Z6, Z7))
    curve_list.append(Segment(Z7, Z8))
    curve_list.append(Segment(Z8, Z9))
    curve_list.append(Segment(Z9, Z1))
    point_ref = (Z11 + Z4 + Z6 + Z10) / 4
    S4 = SurfLine(line_list=curve_list, label="Hole" + st, point_ref=point_ref)

    if self.magnet_0:
        S1.label = S1.label + "_R0_T0_S0"  # Hole
        S3.label = S3.label + "_R0_T1_S0"  # Hole
        surf_list = [S1, S2, S3]
    else:
        S4.label = S4.label + "_R0_T0_S0"  # Hole
        surf_list = [S4]

    # Apply the transformations
    for surf in surf_list:
        surf.rotate(alpha)
        surf.translate(delta)

    return surf_list



import platform
import shutil
import sys
from json import dump, load
from logging import getLogger
from os import makedirs
from os.path import abspath, dirname, isdir, isfile, join, normpath, realpath

from matplotlib.cm import get_cmap, register_cmap
from matplotlib.colors import ListedColormap
from matplotlib import font_manager
from numpy import load as np_load

from ..default_config_dict import default_config_dict


def save_config_dict(config_dict):
    """update the config file with config_dict values

    Parameters
    ----------
    config_dict : dict
        new values to put in the config file
    """
    # dynamic import to avoid loop
    module = __import__(
        "pyleecan.definitions",
        globals=globals(),
        locals=locals(),
        fromlist=["CONF_PATH"],
        level=0,
    )
    CONF_PATH = module.CONF_PATH
    with open(CONF_PATH, "w") as config_file:
        dump(config_dict, config_file, sort_keys=True, indent=4, separators=(",", ": "))


def init_user_dir():
    """Initialize the USER DIR with the default files"""
    # dynamic import to avoid loop
    module = __import__(
        "pyleecan.definitions",
        globals=globals(),
        locals=locals(),
        fromlist=["USER_DIR", "MAIN_DIR"],
        level=0,
    )
    USER_DIR = module.USER_DIR
    MAIN_DIR = module.MAIN_DIR

    logger = getLogger("Pyleecan")

    # Create the main folder
    if not isdir(USER_DIR):
        makedirs(USER_DIR)
        # Copy initial DATA
        logger.debug("Initialization of USER_DIR in " + USER_DIR)

    # Data initialization
    mach_path = join(USER_DIR, "Machine")
    if not isdir(mach_path):
        shutil.copytree(join(MAIN_DIR, "Data", "Machine"), mach_path)
        logger.debug("Initialization USER_DIR Machines in " + mach_path)

    mat_path = join(USER_DIR, "Material")
    if not isdir(mat_path):
        shutil.copytree(join(MAIN_DIR, "Data", "Material"), mat_path)
        logger.debug("Initialization USER_DIR Materials in " + mat_path)

    plot_path = join(USER_DIR, "Plot")
    if not isdir(plot_path):
        shutil.copytree(join(MAIN_DIR, "Data", "Plot"), plot_path)
        logger.debug("Initialization USER_DIR Plot in " + plot_path)

    gui_path = join(USER_DIR, "GUI")
    if not isdir(gui_path):
        shutil.copytree(join(MAIN_DIR, "Data", "GUI"), gui_path)
        logger.debug("Initialization USER_DIR GUI in " + gui_path)


def get_config_dict():
    """Return the config dict (create the default one if needed)

    Returns
    -------
    config_dict: dict
        Dictionnary gather the parameters of the software
    """
    # dynamic import to avoid loop
    module = __import__(
        "pyleecan.definitions",
        globals=globals(),
        locals=locals(),
        fromlist=["USER_DIR", "CONF_PATH"],
        level=0,
    )
    USER_DIR = module.USER_DIR
    CONF_PATH = module.CONF_PATH

    logger = getLogger("Pyleecan")

    # Initialization to make sure all the parameters exist
    init_user_dir()
    config_dict = default_config_dict.copy()
    if isfile(CONF_PATH):
        with open(CONF_PATH, "r") as config_file:
            update_dict(source=config_dict, update=load(config_file))
    else:
        logger.debug("Creating missing config_dict in " + CONF_PATH)
    save_config_dict(config_dict)

    # Load the color_dict
    color_path = join(USER_DIR, "Plot", config_dict["PLOT"]["COLOR_DICT_NAME"])
    def_color_path = join(USER_DIR, "Plot", "pyleecan_color.json")
    if not isfile(color_path):  # Default colors
        logger.warning(
            "Unable to load colors from " + color_path + ", using default colors"
        )
        color_path = join(USER_DIR, "Plot", "pyleecan_color.json")

    # Load default to make sure that the keys are all there
    with open(def_color_path, "r") as color_file:
        config_dict["PLOT"]["COLOR_DICT"] = load(color_file)
    if color_path != def_color_path:
        with open(color_path, "r") as color_file:
            update_dict(
                source=config_dict["PLOT"]["COLOR_DICT"], update=load(color_file)
            )

    # Register the colormap
    cmap_name = config_dict["PLOT"]["COLOR_DICT"]["COLOR_MAP"]
    if "." not in cmap_name:
        cmap_path = join(USER_DIR, "Plot", cmap_name) + ".npy"
    try:
        get_cmap(name=cmap_name)
    except:
        if not isfile(cmap_path):  # Default colormap
            config_dict["PLOT"]["COLOR_DICT"]["COLOR_MAP"] = "RdBu_r"
        else:
            cmap = np_load(cmap_path)
            cmp = ListedColormap(cmap)
            register_cmap(name=config_dict["PLOT"]["COLOR_DICT"]["COLOR_MAP"], cmap=cmp)

    # Check if font is available
    font_name = config_dict["PLOT"]["FONT_NAME"]
    if font_name not in [f.name for f in font_manager.fontManager.ttflist]:
        logger.warning(
            "WARNING: "
            + font_name
            + "font not available. Try: matplotlib.font_manager._rebuild()"
        )
        config_dict["PLOT"]["FONT_NAME"] = "Arial"  # Default font

    return config_dict


def update_dict(source, update):
    for key, value in update.items():
        if isinstance(value, dict):
            source[key] = update_dict(source.get(key, {}), value)
        else:
            source[key] = value
    return source



def has_magnet(self):
    """Return if the Hole has magnets

    Parameters
    ----------
    self : HoleM57
        A HoleM57 object

    Returns
    -------
    has_magnet : bool
        True if the magnets are not None
    """

    return self.magnet_0 is not None or self.magnet_1 is not None



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Machine/MachineUD.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Machine/MachineUD
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .Machine import Machine

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Machine.MachineUD.build_geometry import build_geometry
except ImportError as error:
    build_geometry = error

try:
    from ..Methods.Machine.MachineUD.is_synchronous import is_synchronous
except ImportError as error:
    is_synchronous = error


from ._check import InitUnKnowClassError
from .Lamination import Lamination
from .Frame import Frame
from .Shaft import Shaft


class MachineUD(Machine):
    """User defined Machine with multiple Laminations"""

    VERSION = 1

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Machine.MachineUD.build_geometry
    if isinstance(build_geometry, ImportError):
        build_geometry = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use MachineUD method build_geometry: " + str(build_geometry)
                )
            )
        )
    else:
        build_geometry = build_geometry
    # cf Methods.Machine.MachineUD.is_synchronous
    if isinstance(is_synchronous, ImportError):
        is_synchronous = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use MachineUD method is_synchronous: " + str(is_synchronous)
                )
            )
        )
    else:
        is_synchronous = is_synchronous
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        lam_list=-1,
        is_sync=True,
        frame=-1,
        shaft=-1,
        name="default_machine",
        desc="",
        type_machine=1,
        logger_name="Pyleecan.Machine",
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "lam_list" in list(init_dict.keys()):
                lam_list = init_dict["lam_list"]
            if "is_sync" in list(init_dict.keys()):
                is_sync = init_dict["is_sync"]
            if "frame" in list(init_dict.keys()):
                frame = init_dict["frame"]
            if "shaft" in list(init_dict.keys()):
                shaft = init_dict["shaft"]
            if "name" in list(init_dict.keys()):
                name = init_dict["name"]
            if "desc" in list(init_dict.keys()):
                desc = init_dict["desc"]
            if "type_machine" in list(init_dict.keys()):
                type_machine = init_dict["type_machine"]
            if "logger_name" in list(init_dict.keys()):
                logger_name = init_dict["logger_name"]
        # Set the properties (value check and convertion are done in setter)
        self.lam_list = lam_list
        self.is_sync = is_sync
        # Call Machine init
        super(MachineUD, self).__init__(
            frame=frame,
            shaft=shaft,
            name=name,
            desc=desc,
            type_machine=type_machine,
            logger_name=logger_name,
        )
        # The class is frozen (in Machine init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        MachineUD_str = ""
        # Get the properties inherited from Machine
        MachineUD_str += super(MachineUD, self).__str__()
        if len(self.lam_list) == 0:
            MachineUD_str += "lam_list = []" + linesep
        for ii in range(len(self.lam_list)):
            tmp = self.lam_list[ii].__str__().replace(linesep, linesep + "\t") + linesep
            MachineUD_str += "lam_list[" + str(ii) + "] =" + tmp + linesep + linesep
        MachineUD_str += "is_sync = " + str(self.is_sync) + linesep
        return MachineUD_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from Machine
        if not super(MachineUD, self).__eq__(other):
            return False
        if other.lam_list != self.lam_list:
            return False
        if other.is_sync != self.is_sync:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from Machine
        diff_list.extend(super(MachineUD, self).compare(other, name=name))
        if (other.lam_list is None and self.lam_list is not None) or (
            other.lam_list is not None and self.lam_list is None
        ):
            diff_list.append(name + ".lam_list None mismatch")
        elif self.lam_list is None:
            pass
        elif len(other.lam_list) != len(self.lam_list):
            diff_list.append("len(" + name + ".lam_list)")
        else:
            for ii in range(len(other.lam_list)):
                diff_list.extend(
                    self.lam_list[ii].compare(
                        other.lam_list[ii], name=name + ".lam_list[" + str(ii) + "]"
                    )
                )
        if other._is_sync != self._is_sync:
            diff_list.append(name + ".is_sync")
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from Machine
        S += super(MachineUD, self).__sizeof__()
        if self.lam_list is not None:
            for value in self.lam_list:
                S += getsizeof(value)
        S += getsizeof(self.is_sync)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        # Get the properties inherited from Machine
        MachineUD_dict = super(MachineUD, self).as_dict(**kwargs)
        if self.lam_list is None:
            MachineUD_dict["lam_list"] = None
        else:
            MachineUD_dict["lam_list"] = list()
            for obj in self.lam_list:
                if obj is not None:
                    MachineUD_dict["lam_list"].append(obj.as_dict(**kwargs))
                else:
                    MachineUD_dict["lam_list"].append(None)
        MachineUD_dict["is_sync"] = self.is_sync
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        MachineUD_dict["__class__"] = "MachineUD"
        return MachineUD_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.lam_list = None
        self.is_sync = None
        # Set to None the properties inherited from Machine
        super(MachineUD, self)._set_None()

    def _get_lam_list(self):
        """getter of lam_list"""
        if self._lam_list is not None:
            for obj in self._lam_list:
                if obj is not None:
                    obj.parent = self
        return self._lam_list

    def _set_lam_list(self, value):
        """setter of lam_list"""
        if type(value) is list:
            for ii, obj in enumerate(value):
                if type(obj) is dict:
                    class_obj = import_class(
                        "pyleecan.Classes", obj.get("__class__"), "lam_list"
                    )
                    value[ii] = class_obj(init_dict=obj)
                if value[ii] is not None:
                    value[ii].parent = self
        if value == -1:
            value = list()
        check_var("lam_list", value, "[Lamination]")
        self._lam_list = value

    lam_list = property(
        fget=_get_lam_list,
        fset=_set_lam_list,
        doc=u"""List of Lamination

        :Type: [Lamination]
        """,
    )

    def _get_is_sync(self):
        """getter of is_sync"""
        return self._is_sync

    def _set_is_sync(self, value):
        """setter of is_sync"""
        check_var("is_sync", value, "bool")
        self._is_sync = value

    is_sync = property(
        fget=_get_is_sync,
        fset=_set_is_sync,
        doc=u"""True if the machine should be handled as a Synchronous machine

        :Type: bool
        """,
    )



from numpy import zeros
from os.path import splitext
from ...Functions.FEMM.update_FEMM_simulation import update_FEMM_simulation
from ...Functions.FEMM.comp_FEMM_Phi_wind import comp_FEMM_Phi_wind


def solve_FEMM(obj, femm, output, sym, FEMM_dict):

    L1 = output.simu.machine.stator.comp_length()
    Nt_tot = obj.Nt_tot  # Number of time step
    is_internal_rotor = output.simu.machine.rotor.is_internal

    if (
        hasattr(output.simu.machine.stator, "winding")
        and output.simu.machine.stator.winding is not None
    ):
        qs = output.simu.machine.stator.winding.qs  # Winding phase number
        Npcpp = output.simu.machine.stator.winding.Npcpp
        Phi_wind_stator = zeros((Nt_tot, qs))
    else:
        Phi_wind_stator = None

    # Interpolate current on electric model time axis
    # Get stator current from elec out
    Is = output.elec.comp_I_mag(time=output.elec.Time.values, is_stator=True)

    # Get rotor current from elec out
    Ir = output.elec.comp_I_mag(time=output.elec.Time.values, is_stator=False)

    # Get rotor angular position
    angle_rotor = output.get_angle_rotor()[0:Nt_tot]

    # Create the mesh
    femm.mi_createmesh()

    # Compute the data for each time step
    for ii in range(Nt_tot):
        # Update rotor position and currents
        update_FEMM_simulation(
            femm,
            FEMM_dict["circuits"],
            is_internal_rotor,
            obj.is_sliding_band,
            angle_rotor,
            Is,
            Ir,
            ii,
        )
        # try "previous solution" for speed up of FEMM calculation
        if obj.is_sliding_band:
            try:
                base = FEMM_dict["path_save"]
                ans_file = splitext(base)[0] + ".ans"
                femm.mi_setprevious(ans_file, 0)
            except:
                pass

        # Run the computation
        femm.mi_analyze()
        femm.mi_loadsolution()

        if Phi_wind_stator is not None:
            # Phi_wind computation
            Phi_wind_stator[ii, :] = comp_FEMM_Phi_wind(
                femm,
                qs,
                Npcpp,
                is_stator=True,
                Lfemm=FEMM_dict["Lfemm"],
                L1=L1,
                sym=sym,
            )

    return Phi_wind_stator



# -*- coding: utf-8 -*-
from ....Methods.Import import ImportError


class GenSinEmptyError(ImportError):
    """Raised when the sin_list is empty"""

    pass


class GenSinDimError(ImportError):
    """Raised when the ImportGenVectSin of the sin_list has different size (N)"""

    pass


class GenSinTransposeError(ImportError):
    """Raised when the ImportGenVectSin of the sin_list has different is_transpose"""

    pass


class InitSinMatDimError(ImportError):
    """Raised when the init_vector is called with wrong arguments"""

    pass



# -*- coding: utf-8 -*-
"""File generated according to PWSlot26/gen_list.json
WARNING! All changes made in this file will be lost!
"""
from pyleecan.GUI.Dialog.DMachineSetup.SWSlot.PWSlot26.Ui_PWSlot26 import Ui_PWSlot26


class Gen_PWSlot26(Ui_PWSlot26):
    def setupUi(self, PWSlot26):
        """Abstract class to update the widget according to the csv doc"""
        Ui_PWSlot26.setupUi(self, PWSlot26)
        # Setup of in_W0
        txt = self.tr(u"""Slot isthmus width.""")
        self.in_W0.setWhatsThis(txt)
        self.in_W0.setToolTip(txt)

        # Setup of lf_W0
        self.lf_W0.validator().setBottom(0)
        txt = self.tr(u"""Slot isthmus width.""")
        self.lf_W0.setWhatsThis(txt)
        self.lf_W0.setToolTip(txt)

        # Setup of in_H0
        txt = self.tr(u"""Slot isthmus height.""")
        self.in_H0.setWhatsThis(txt)
        self.in_H0.setToolTip(txt)

        # Setup of lf_H0
        self.lf_H0.validator().setBottom(0)
        txt = self.tr(u"""Slot isthmus height.""")
        self.lf_H0.setWhatsThis(txt)
        self.lf_H0.setToolTip(txt)

        # Setup of in_H1
        txt = self.tr(u"""Slot depth """)
        self.in_H1.setWhatsThis(txt)
        self.in_H1.setToolTip(txt)

        # Setup of lf_H1
        self.lf_H1.validator().setBottom(0)
        txt = self.tr(u"""Slot depth """)
        self.lf_H1.setWhatsThis(txt)
        self.lf_H1.setToolTip(txt)

        # Setup of in_R1
        txt = self.tr(u"""Slot edge radius""")
        self.in_R1.setWhatsThis(txt)
        self.in_R1.setToolTip(txt)

        # Setup of lf_R1
        self.lf_R1.validator().setBottom(0)
        txt = self.tr(u"""Slot edge radius""")
        self.lf_R1.setWhatsThis(txt)
        self.lf_R1.setToolTip(txt)

        # Setup of in_R2
        txt = self.tr(u"""Slot top radius""")
        self.in_R2.setWhatsThis(txt)
        self.in_R2.setToolTip(txt)

        # Setup of lf_R2
        self.lf_R2.validator().setBottom(0)
        txt = self.tr(u"""Slot top radius""")
        self.lf_R2.setWhatsThis(txt)
        self.lf_R2.setToolTip(txt)



# -*-- coding: utf-8 -*
from numpy import angle, exp

from ....Classes.Arc1 import Arc1
from ....Classes.Segment import Segment


def get_lines(self):
    """return the list of lines that delimits the PolarArc

    Parameters
    ----------
    self : PolarArc
        a PolarArc object

    Returns
    -------
    line_list: list
        List of line need to draw the slot (2 Segment + 2 Arc1)
    """
    # check if the PolarArc is correct
    self.check()

    Z_ref = self.point_ref
    center = Z_ref * exp(-1j * angle(Z_ref))
    H = self.height
    A = self.angle
    # the points of the PolarArc
    Z2 = (center - (H / 2)) * exp(1j * (-(A / 2))) * exp(1j * angle(Z_ref))
    Z3 = (center + (H / 2)) * exp(1j * (-(A / 2))) * exp(1j * angle(Z_ref))
    Z4 = (center + (H / 2)) * exp(1j * (A / 2)) * exp(1j * angle(Z_ref))
    Z1 = (center - (H / 2)) * exp(1j * (A / 2)) * exp(1j * angle(Z_ref))

    # Lines that delimit the PolarArc
    line1 = Arc1(Z1, Z2, -abs(Z1), is_trigo_direction=False)
    line2 = Segment(Z2, Z3)
    line3 = Arc1(Z3, Z4, abs(Z3))
    line4 = Segment(Z4, Z1)

    return [line1, line2, line3, line4]



# -*- coding: utf-8 -*-

from numpy import exp, pi

from ....Classes.Trapeze import Trapeze
from ....Methods.Slot.VentilationTrap import *


def build_geometry(self, sym=1, alpha=0, delta=0, is_stator=True):
    """Compute the curve needed to plot the ventilations

    Parameters
    ----------
    self : VentilationTrap
        A VentilationTrap object
    sym : int
        Symetry to apply 2 = half the machine (Default value = 1 => full machine)
    alpha : float
        Angle for rotation (Default value = 0) [rad]
    delta : complex
        Complex for translation (Default value = 0)
    is_stator : bool
        True if ventilation is on the stator and 0 on the rotor (Default value = True)

    Returns
    -------
    surf_list: list
        A list of Trapeze
    """

    if is_stator:
        st = "Stator"
    else:
        st = "Rotor"

    # checking if the param have good type
    if not (isinstance(sym, int)) or sym <= 0:
        raise TrapezeBuildGeometryError("The parameter 'sym' must be an integer > 0")
    if type(alpha) not in [int, float]:
        raise TrapezeBuildGeometryError("The parameter 'alpha' must be an int or float")
    if type(delta) not in [complex, int, float]:
        raise TrapezeBuildGeometryError(
            "The parameter 'delta' must be a complex or float or int number"
        )

    surf_list = list()
    assert self.Zh % sym == 0
    Zh = self.Zh // sym
    # For every Trapeze
    for ii in range(Zh):
        Zc = (self.H0 + (self.D0 / 2)) * exp(
            1j * ((self.Alpha0 + ii * 2 * pi / self.Zh) + (pi / self.Zh + alpha))
        )
        Zc += delta
        surf_list.append(
            Trapeze(
                point_ref=Zc,
                label="Ventilation_" + st + "_" + str(ii),
                height=self.D0,
                W1=self.W1,
                W2=self.W2,
            )
        )
    return surf_list



# -*- coding: utf-8 -*-


def comp_width(self):
    """Compute the width of the conductor

    Parameters
    ----------
    self : CondType21
        A CondType21 object

    Returns
    -------
    W: float
        Width of the conductor (with insulation) [m]

    """

    return self.Wbar + 2 * self.Wins



# -*- coding: utf-8 -*-

from .....Functions.Plot.plot_A_time_space import (
    plot_A_time_space as plot_A_time_space_fct,
)
from SciDataTool import VectorField


def plot_A_time_space(
    self,
    Data_str,
    is_deg=True,
    is_elecorder=False,
    is_spaceorder=False,
    freq_max=20000,
    r_max=100,
    z_max=None,
    is_norm=False,
    unit="SI",
    save_path=None,
    is_auto_ticks=True,
    component_list=None,
    is_show_fig=None,
):
    """Plots a field as a function of time and space (angle)

    Parameters
    ----------
    self : Output
        an Output object
    Data_str : str
        name of the Data Object to plot (e.g. "mag.Br")
    is_deg : bool
        boolean indicating if the angle must be converted to degrees
    is_elecorder : bool
        boolean indicating if we want to use the electrical order for the fft axis
    is_spaceorder : bool
        boolean indicating if we want to use the spatial order for the fft axis
    freq_max : float
        maximum value of the frequency for the fft axis
    r_max : int
        maximum value of the wavenumber for the fft axis
    z_max : float
        maximum value for the field
    is_norm : bool
        boolean indicating if the field must be normalized
    unit : str
        unit in which to plot the field
    colormap : colormap object
        colormap prescribed by user
    save_path : str
        full path including folder, name and extension of the file to save if save_path is not None
    is_auto_ticks : bool
        in fft, adjust ticks to freqs and wavenumbers (deactivate if too close)
    component_list : list
        list of component names to plot in separate figures
    is_show_fig : bool
        True to show figure after plot
    """

    # Get Data object names
    phys = getattr(self, Data_str.split(".")[0])
    data = getattr(phys, Data_str.split(".")[1])

    # Call the plot function
    if isinstance(data, VectorField):
        if component_list is None:  # default: extract all components
            component_list = data.components.keys()
        for comp in component_list:
            plot_A_time_space_fct(
                data.components[comp],
                is_deg=is_deg,
                is_elecorder=is_elecorder,
                is_spaceorder=is_spaceorder,
                freq_max=freq_max,
                r_max=r_max,
                z_max=z_max,
                is_norm=is_norm,
                unit=unit,
                save_path=save_path,
                is_auto_ticks=is_auto_ticks,
                is_show_fig=is_show_fig,
            )

    else:
        plot_A_time_space_fct(
            data,
            is_deg=is_deg,
            is_elecorder=is_elecorder,
            is_spaceorder=is_spaceorder,
            freq_max=freq_max,
            r_max=r_max,
            z_max=z_max,
            is_norm=is_norm,
            unit=unit,
            save_path=save_path,
            is_auto_ticks=is_auto_ticks,
            is_show_fig=is_show_fig,
        )



# -*- coding: utf-8 -*-

from ....Methods.Simulation.Input import InputError


def run(self):
    """Run the Electrical module"""
    if self.parent is None:
        raise InputError(
            "ERROR: The Electrical object must be in a Simulation object to run"
        )
    if self.parent.parent is None:
        raise InputError(
            "ERROR: The Simulation object must be in an Output object to run"
        )

    self.get_logger().info("Starting Electric module")

    output = self.parent.parent

    if self.eec is not None:

        # # Generate drive
        # self.eec.gen_drive(output)

        # Compute parameters of the electrical equivalent circuit
        self.eec.comp_parameters(output)

        # Solve the electrical equivalent circuit
        self.eec.solve_EEC(output)

        # Compute losses due to Joule effects
        self.eec.comp_joule_losses(output)

        # Compute electromagnetic power
        self.comp_power(output)

        # Compute torque
        self.comp_torque(output)



from ...Classes.SurfLine import SurfLine
from ...Classes.Segment import Segment
from ...Classes.Arc import Arc
from ...Classes.Arc2 import Arc2


class SurfaceError(Exception):
    pass


def create_surface(line_list):
    """Create a surface from a list of Line

    Parameters
    ----------
    line_list: list
        list of line forming the surface

    Returns
    -------
    surface: SurfLine
    """
    ordered_list = [line_list.pop(0)]
    first_element = ordered_list[0]
    if isinstance(first_element, (Arc, Segment)):
        end = first_element.get_end()
    else:
        raise NotImplementedError(
            "The function create_surface only handles Arc and Segment lines."
        )
    surface_points = []
    for _ in range(len(line_list)):
        surface_points.append(end)
        found = False
        for line in line_list[:]:
            if abs(end - line.get_begin()) < 1e-09:
                new_line = line.copy()
                new_line.begin = end
                ordered_list.append(new_line)
                end = new_line.get_end()
                found = True
                line_list.remove(line)
                break
            elif abs(end - line.get_end()) < 1e-09:
                if isinstance(line, Segment):
                    new_line = Segment(begin=end, end=(line.begin))
                elif isinstance(line, Arc):
                    new_line = Arc2(
                        begin=end, center=(line.get_center()), angle=(-line.get_angle())
                    )
                else:
                    raise NotImplementedError(
                        "The function create_surface only handles Arc and Segment lines."
                    )
                ordered_list.append(new_line)
                end = new_line.get_end()
                line_list.remove(line)
                found = True
                break

        assert found, SurfaceError("Cannot find the next line of the surface")

    ordered_list[0].begin = ordered_list[(-1)].get_end()
    return SurfLine(
        line_list=ordered_list, point_ref=(sum(surface_points) / len(surface_points))
    )



# -*- coding: utf-8 -*-

import pyvista as pv
from meshio import read


def get_mesh_pv(self, indices=None):
    """Return the pyvista mesh object (or submesh).

    Parameters
    ----------
    self : MeshVTK
        a MeshVTK object
    indices : list
        list of the points to extract (optional)

    Returns
    -------
    mesh : pyvista.core.pointset.UnstructuredGrid
        a pyvista UnstructuredGrid object
    """

    # Already available => Return
    if self.mesh is not None:
        # Extract submesh
        if indices is not None:
            mesh = self.mesh.extract_points(indices)
        return mesh

    # Read mesh file
    else:
        if self.format != "vtk" and self.format != "vtu":
            # Write vtk files with meshio
            # in case replace whitespace in point data keys since vtk doesn't like it
            mesh = read(self.path + "/" + self.name + "." + self.format)
            mesh.point_data = {
                k.replace(" ", "_"): v for k, v in mesh.point_data.items()
            }
            mesh.write(self.path + "/" + self.name + ".vtk")
            use_this_format = ".vtk"
        else:
            use_this_format = "." + self.format

        # Read .vtk file with pyvista
        mesh = pv.read(self.path + "/" + self.name + use_this_format)

        # Extract submesh
        if indices is not None:
            mesh = mesh.extract_points(indices)

        if self.is_pyvista_mesh:
            self.mesh = mesh

        return mesh



# -*- coding: utf-8 -*-

import PySide2.QtCore
from numpy import pi
from PySide2.QtCore import Signal
from PySide2.QtWidgets import QWidget

from ......Classes.SlotM17 import SlotM17
from ......GUI import gui_option
from ......GUI.Dialog.DMachineSetup.SMSlot.PMSlot17.Gen_PMSlot17 import Gen_PMSlot17
from ......Methods.Slot.Slot import SlotCheckError

translate = PySide2.QtCore.QCoreApplication.translate


class PMSlot17(Gen_PMSlot17, QWidget):
    """Page to set the Slot Magnet Type 17"""

    # Signal to DMachineSetup to know that the save popup is needed
    saveNeeded = Signal()
    # Information for Slot combobox
    slot_name = "Cylindrical magnet (no lamination)"
    slot_type = SlotM17

    def __init__(self, lamination=None):
        """Initialize the widget according to lamination

        Parameters
        ----------
        self : PMSlot17
            A PMSlot17 widget
        lamination : Lamination
            current lamination to edit
        """

        # Build the interface according to the .ui file
        QWidget.__init__(self)
        self.setupUi(self)
        self.lamination = lamination
        self.slot = lamination.slot

        # Set FloatEdit unit
        self.lf_Lmag.unit = "m"
        # Set unit name (m ou mm)
        wid_list = [
            self.unit_Lmag,
        ]
        for wid in wid_list:
            wid.setText("[" + gui_option.unit.get_m_name() + "]")

        # Fill the fields with the machine values (if they're filled)
        if self.lamination.magnet.Lmag is None:
            self.lamination.magnet.Lmag = self.lamination.L1
        self.lf_Lmag.setValue(self.lamination.magnet.Lmag)

        # Display the main output of the slot (surface, height...)
        self.w_out.comp_output()

        # Connect the signal
        self.lf_Lmag.editingFinished.connect(self.set_Lmag)

    def set_Lmag(self):
        """Signal to update the value of Lmag according to the line edit

        Parameters
        ----------
        self : PMSlot17
            A PMSlot17 object
        """
        self.lamination.magnet.Lmag = self.lf_Lmag.value()
        self.w_out.comp_output()
        # Notify the machine GUI that the machine has changed
        self.saveNeeded.emit()

    @staticmethod
    def check(lam):
        """Check that the current machine have all the needed field set

        Parameters
        ----------
        lam: LamSlotMag
            Lamination to check

        Returns
        -------
        error: str
            Error message (return None if no error)
        """

        # Check that everything is set
        if lam.magnet.Lmag is None:
            return "You must set Lmag !"

        # Constraints
        try:
            lam.slot.check()
        except SlotCheckError as error:
            return str(error)

        # Output
        try:
            yoke_height = lam.comp_height_yoke()
        except Exception as error:
            return translate("Unable to compute yoke height:", "PMSlot17") + str(error)



# -*- coding: utf-8 -*-

from numpy import exp, pi, cos, sin, tan

from ....Classes.Segment import Segment
from ....Classes.Arc1 import Arc1
from ....Classes.SurfLine import SurfLine


def build_geometry(self, alpha=0, delta=0, is_simplified=False):
    """Compute the curve (Line) needed to plot the Hole.
    The ending point of a curve is the starting point of the next curve in
    the list

    Parameters
    ----------
    self : HoleM51
        A HoleM51 object
    alpha : float
        Angle to rotate the hole (Default value = 0) [rad]
    delta : complex
        Complex to translate the hole (Default value = 0)
    is_simplified : bool
       True to avoid line superposition

    Returns
    -------
    surf_list: list
        List of SurfLine needed to draw the HoleM51

    """
    if self.get_is_stator():  # check if the slot is on the stator
        st = "_Stator"
    else:
        st = "_Rotor"
    Rext = self.get_Rext()

    # Get all the points
    point_dict = self._comp_point_coordinate()
    Z1 = point_dict["Z1"]
    Z2 = point_dict["Z2"]
    Z3 = point_dict["Z3"]
    Z4 = point_dict["Z4"]
    Z5 = point_dict["Z5"]
    Z6 = point_dict["Z6"]
    Z8 = point_dict["Z8"]
    Z9 = point_dict["Z9"]
    Z11 = point_dict["Z11"]
    Z13 = point_dict["Z13"]
    Z14 = point_dict["Z14"]
    Z16 = point_dict["Z16"]
    Z18 = point_dict["Z18"]
    Z19 = point_dict["Z19"]
    Z21 = point_dict["Z21"]
    Z22 = point_dict["Z22"]
    Z23 = point_dict["Z23"]
    Z24 = point_dict["Z24"]
    Z25 = point_dict["Z25"]
    Z26 = point_dict["Z26"]

    surf_list = list()
    # Create all the surfaces for all the cases
    # Air surface bore around magnet_0
    curve_list = list()
    curve_list.append(Arc1(Z21, Z22, Rext - self.H1))
    curve_list.append(Segment(Z22, Z23))
    curve_list.append(Segment(Z23, Z19))
    curve_list.append(Segment(Z19, Z21))
    point_ref = (Z21 + Z22 + Z23 + Z19) / 4
    S1 = SurfLine(line_list=curve_list, label="Hole" + st, point_ref=point_ref)

    # Surface for magnet_0
    curve_list = list()
    if is_simplified:
        curve_list.append(Segment(Z24, Z18))
        curve_list.append(Segment(Z19, Z23))
    else:
        curve_list.append(Segment(Z24, Z18))
        curve_list.append(Segment(Z18, Z19))
        curve_list.append(Segment(Z19, Z23))
        curve_list.append(Segment(Z23, Z24))
    point_ref = (Z24 + Z18 + Z19 + Z23) / 4
    # Defining type of magnetization of the magnet
    if self.magnet_0:
        if self.magnet_0.type_magnetization == 0:
            type_mag = "_Radial"
        else:
            type_mag = "_Parallel"
    else:
        type_mag = "None"
    magnet_label = "HoleMagnet" + st + type_mag + "_N_R0_T0_S0"
    S2 = SurfLine(line_list=curve_list, label=magnet_label, point_ref=point_ref)

    # Air surface between magnet_0 and magnet_1
    curve_list = list()
    curve_list.append(Segment(Z24, Z25))
    curve_list.append(Segment(Z25, Z26))
    curve_list.append(Segment(Z26, Z14))
    curve_list.append(Segment(Z14, Z16))
    curve_list.append(Segment(Z16, Z18))
    curve_list.append(Segment(Z18, Z24))
    point_ref = (Z25 + Z16) / 2
    S3 = SurfLine(line_list=curve_list, label="Hole" + st, point_ref=point_ref)

    # Surface for magnet_1
    curve_list = list()
    if is_simplified:
        curve_list.append(Segment(Z1, Z13))
        curve_list.append(Segment(Z14, Z26))
    else:
        curve_list.append(Segment(Z26, Z1))
        curve_list.append(Segment(Z1, Z13))
        curve_list.append(Segment(Z13, Z14))
        curve_list.append(Segment(Z14, Z26))
    point_ref = (Z26 + Z1 + Z13 + Z14) / 4
    # Defining type of magnetization of the magnet
    if self.magnet_1:
        if self.magnet_1.type_magnetization == 0:
            type_mag = "_Radial"
        else:
            type_mag = "_Parallel"
    else:
        type_mag = "None"
    magnet_label = "HoleMagnet" + st + type_mag + "_N_R0_T1_S0"
    S4 = SurfLine(line_list=curve_list, label=magnet_label, point_ref=point_ref)

    # Air surface between magnet_1 and magnet_2
    curve_list = list()
    curve_list.append(Segment(Z1, Z2))
    curve_list.append(Segment(Z2, Z3))
    curve_list.append(Segment(Z3, Z9))
    curve_list.append(Segment(Z9, Z11))
    curve_list.append(Segment(Z11, Z13))
    curve_list.append(Segment(Z13, Z1))
    point_ref = (Z11 + Z2) / 2
    S5 = SurfLine(line_list=curve_list, label="Hole" + st, point_ref=point_ref)

    # Surface for magnet_2
    curve_list = list()
    if is_simplified:
        curve_list.append(Segment(Z4, Z8))
        curve_list.append(Segment(Z9, Z3))
    else:
        curve_list.append(Segment(Z4, Z8))
        curve_list.append(Segment(Z8, Z9))
        curve_list.append(Segment(Z9, Z3))
        curve_list.append(Segment(Z3, Z4))
    point_ref = (Z4 + Z8 + Z9 + Z3) / 4
    # Defining type of magnetization of the magnet
    if self.magnet_2:
        if self.magnet_2.type_magnetization == 0:
            type_mag = "_Radial"
        else:
            type_mag = "_Parallel"
    else:
        type_mag = "None"
    magnet_label = "HoleMagnet" + st + type_mag + "_N_R0_T2_S0"
    S6 = SurfLine(line_list=curve_list, label=magnet_label, point_ref=point_ref)

    # Air surface bore around magnet_2
    curve_list = list()
    curve_list.append(Arc1(Z5, Z6, Rext - self.H1))
    curve_list.append(Segment(Z6, Z8))
    curve_list.append(Segment(Z8, Z4))
    curve_list.append(Segment(Z4, Z5))
    point_ref = (Z4 + Z5 + Z6 + Z8) / 4
    S7 = SurfLine(line_list=curve_list, label="Hole" + st, point_ref=point_ref)

    # Air surface bore around magnet_2 (no magnet_2 and magnet_1)
    curve_list = list()
    curve_list.append(Arc1(Z5, Z6, Rext - self.H1))
    curve_list.append(Segment(Z6, Z11))
    curve_list.append(Segment(Z11, Z13))
    curve_list.append(Segment(Z13, Z1))
    curve_list.append(Segment(Z1, Z2))
    curve_list.append(Segment(Z2, Z5))
    point_ref = (Z2 + Z11) / 2
    S8 = SurfLine(line_list=curve_list, label="Hole" + st, point_ref=point_ref)

    # Air surface bore around magnet_0 (no magnet_0 and magnet_1)
    curve_list = list()
    curve_list.append(Arc1(Z21, Z22, Rext - self.H1))
    curve_list.append(Segment(Z22, Z25))
    curve_list.append(Segment(Z25, Z26))
    curve_list.append(Segment(Z26, Z14))
    curve_list.append(Segment(Z14, Z16))
    curve_list.append(Segment(Z16, Z21))
    point_ref = (Z25 + Z16) / 2
    S9 = SurfLine(line_list=curve_list, label="Hole" + st, point_ref=point_ref)

    # Air surface bore around magnet_1 (no magnet_1 and magnet_0 and magnet_2)
    curve_list = list()
    curve_list.append(Segment(Z24, Z25))
    curve_list.append(Segment(Z25, Z2))
    curve_list.append(Segment(Z2, Z3))
    curve_list.append(Segment(Z3, Z9))
    curve_list.append(Segment(Z9, Z11))
    curve_list.append(Segment(Z11, Z16))
    curve_list.append(Segment(Z16, Z18))
    curve_list.append(Segment(Z18, Z24))
    point_ref = (Z1 + Z13) / 2
    S10 = SurfLine(line_list=curve_list, label="Hole" + st, point_ref=point_ref)

    # Air surface bore around magnet_1 (no magnet_1 and no magnet_0 and magnet_2)
    curve_list = list()
    curve_list.append(Arc1(Z21, Z22, Rext - self.H1))
    curve_list.append(Segment(Z22, Z25))
    curve_list.append(Segment(Z25, Z2))
    curve_list.append(Segment(Z2, Z3))
    curve_list.append(Segment(Z3, Z9))
    curve_list.append(Segment(Z9, Z11))
    curve_list.append(Segment(Z11, Z16))
    curve_list.append(Segment(Z16, Z21))
    point_ref = (Z1 + Z13) / 2
    S11 = SurfLine(line_list=curve_list, label="Hole" + st, point_ref=point_ref)

    # Air surface bore around magnet_1 (no magnet_1 and magnet_0 and no magnet_2)
    curve_list = list()
    curve_list.append(Arc1(Z5, Z6, Rext - self.H1))
    curve_list.append(Segment(Z6, Z11))
    curve_list.append(Segment(Z11, Z16))
    curve_list.append(Segment(Z16, Z18))
    curve_list.append(Segment(Z18, Z24))
    curve_list.append(Segment(Z24, Z25))
    curve_list.append(Segment(Z25, Z2))
    curve_list.append(Segment(Z2, Z5))
    point_ref = (Z1 + Z13) / 2
    S12 = SurfLine(line_list=curve_list, label="Hole" + st, point_ref=point_ref)

    # Air surface No magnet
    curve_list = list()
    curve_list.append(Arc1(Z5, Z6, Rext - self.H1))
    curve_list.append(Segment(Z6, Z11))
    curve_list.append(Segment(Z11, Z16))
    curve_list.append(Segment(Z16, Z21))
    curve_list.append(Arc1(Z21, Z22, Rext - self.H1))
    curve_list.append(Segment(Z22, Z25))
    curve_list.append(Segment(Z25, Z2))
    curve_list.append(Segment(Z2, Z5))
    point_ref = (Z1 + Z13) / 2
    S13 = SurfLine(line_list=curve_list, label="Hole" + st, point_ref=point_ref)

    # Create the surface list by selecting the correct ones
    if self.magnet_0 and self.magnet_1 and self.magnet_2:
        S1.label = S1.label + "_R0_T0_S0"  # Hole
        S3.label = S3.label + "_R0_T1_S0"  # Hole
        S5.label = S5.label + "_R0_T2_S0"  # Hole
        S7.label = S7.label + "_R0_T3_S0"  # Hole
        surf_list = [S1, S2, S3, S4, S5, S6, S7]
    elif not self.magnet_0 and self.magnet_1 and self.magnet_2:
        S9.label = S9.label + "_R0_T0_S0"  # Hole
        S5.label = S5.label + "_R0_T1_S0"  # Hole
        S7.label = S7.label + "_R0_T2_S0"  # Hole
        surf_list = [S9, S4, S5, S6, S7]
    elif self.magnet_0 and not self.magnet_1 and self.magnet_2:
        S1.label = S1.label + "_R0_T0_S0"  # Hole
        S10.label = S10.label + "_R0_T1_S0"  # Hole
        S7.label = S7.label + "_R0_T2_S0"  # Hole
        surf_list = [S1, S2, S10, S6, S7]
    elif not self.magnet_0 and not self.magnet_1 and self.magnet_2:
        S11.label = S11.label + "_R0_T0_S0"  # Hole
        S7.label = S7.label + "_R0_T1_S0"  # Hole
        surf_list = [S11, S6, S7]
    elif self.magnet_0 and self.magnet_1 and not self.magnet_2:
        S1.label = S1.label + "_R0_T0_S0"  # Hole
        S3.label = S3.label + "_R0_T1_S0"  # Hole
        S8.label = S8.label + "_R0_T2_S0"  # Hole
        surf_list = [S1, S2, S3, S4, S8]
    elif not self.magnet_0 and self.magnet_1 and not self.magnet_2:
        S9.label = S9.label + "_R0_T0_S0"  # Hole
        S8.label = S8.label + "_R0_T1_S0"  # Hole
        surf_list = [S9, S4, S8]
    elif self.magnet_0 and not self.magnet_1 and not self.magnet_2:
        S1.label = S1.label + "_R0_T0_S0"  # Hole
        S12.label = S12.label + "_R0_T1_S0"  # Hole
        surf_list = [S1, S2, S12]
    elif not self.magnet_0 and not self.magnet_1 and not self.magnet_2:
        S13.label = S13.label + "_R0_T0_S0"  # Hole
        surf_list = [S13]

    for surf in surf_list:
        surf.rotate(alpha)
        surf.translate(delta)

    return surf_list



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Simulation/DriveWave.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Simulation/DriveWave
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .Drive import Drive

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Simulation.DriveWave.get_wave import get_wave
except ImportError as error:
    get_wave = error


from ._check import InitUnKnowClassError
from .Import import Import


class DriveWave(Drive):
    """Drive to generate a wave according to an Import object"""

    VERSION = 1

    # cf Methods.Simulation.DriveWave.get_wave
    if isinstance(get_wave, ImportError):
        get_wave = property(
            fget=lambda x: raise_(
                ImportError("Can't use DriveWave method get_wave: " + str(get_wave))
            )
        )
    else:
        get_wave = get_wave
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        wave=-1,
        Umax=800,
        Imax=800,
        is_current=False,
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "wave" in list(init_dict.keys()):
                wave = init_dict["wave"]
            if "Umax" in list(init_dict.keys()):
                Umax = init_dict["Umax"]
            if "Imax" in list(init_dict.keys()):
                Imax = init_dict["Imax"]
            if "is_current" in list(init_dict.keys()):
                is_current = init_dict["is_current"]
        # Set the properties (value check and convertion are done in setter)
        self.wave = wave
        # Call Drive init
        super(DriveWave, self).__init__(Umax=Umax, Imax=Imax, is_current=is_current)
        # The class is frozen (in Drive init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        DriveWave_str = ""
        # Get the properties inherited from Drive
        DriveWave_str += super(DriveWave, self).__str__()
        if self.wave is not None:
            tmp = self.wave.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            DriveWave_str += "wave = " + tmp
        else:
            DriveWave_str += "wave = None" + linesep + linesep
        return DriveWave_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from Drive
        if not super(DriveWave, self).__eq__(other):
            return False
        if other.wave != self.wave:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from Drive
        diff_list.extend(super(DriveWave, self).compare(other, name=name))
        if (other.wave is None and self.wave is not None) or (
            other.wave is not None and self.wave is None
        ):
            diff_list.append(name + ".wave None mismatch")
        elif self.wave is not None:
            diff_list.extend(self.wave.compare(other.wave, name=name + ".wave"))
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from Drive
        S += super(DriveWave, self).__sizeof__()
        S += getsizeof(self.wave)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        # Get the properties inherited from Drive
        DriveWave_dict = super(DriveWave, self).as_dict(**kwargs)
        if self.wave is None:
            DriveWave_dict["wave"] = None
        else:
            DriveWave_dict["wave"] = self.wave.as_dict(**kwargs)
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        DriveWave_dict["__class__"] = "DriveWave"
        return DriveWave_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        if self.wave is not None:
            self.wave._set_None()
        # Set to None the properties inherited from Drive
        super(DriveWave, self)._set_None()

    def _get_wave(self):
        """getter of wave"""
        return self._wave

    def _set_wave(self, value):
        """setter of wave"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class("pyleecan.Classes", value.get("__class__"), "wave")
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = Import()
        check_var("wave", value, "Import")
        self._wave = value

        if self._wave is not None:
            self._wave.parent = self

    wave = property(
        fget=_get_wave,
        fset=_set_wave,
        doc=u"""Wave generator

        :Type: Import
        """,
    )



# -*- coding: utf-8 -*-

from ....Classes.OutGeo import OutGeo


def comp_output_geo(self):
    """Compute the main geometry output

    Parameters
    ----------
    self : Machine
        A Machine object

    Returns
    -------
    output: OutGeo
        Main geometry output of the machine

    """

    output = OutGeo()
    output.Wgap_mec = self.comp_width_airgap_mec()
    output.Wgap_mag = self.comp_width_airgap_mag()
    output.Rgap_mec = self.comp_Rgap_mec()
    output.Lgap = self.comp_length_airgap_active()

    output.stator = self.stator.comp_output_geo()
    output.rotor = self.rotor.comp_output_geo()

    return output



# -*- coding: utf-8 -*-


def comp_surface(self):
    """Compute the Hole total surface (by numerical computation).

    Parameters
    ----------
    self : Hole
        A Hole object

    Returns
    -------
    S : float
        Slot total surface [m**2]

    """

    surf_list = self.build_geometry()
    S = 0
    for surf in surf_list:
        S += surf.comp_surface()

    return S



# -*- coding: utf-8 -*-

from ....Methods.Slot.Slot import SlotCheckError
from ....Methods.Slot.SlotW61 import *


def check(self):
    """Check that the SlotW61 object is correct

    Parameters
    ----------
    self : SlotW61
        A SlotW61 object

    Returns
    -------
    None
    Raises
    -------
    S61_InnerCheckError
       Slot 61 is for inner lamination only
    S61_W21CheckError
        You must have W2 < W1
    S61_WindHError
        You must have W3+W4 < H2
    S61_WindWError
        You must have W3 < (W1-W2)/2
    """

    if self.is_outwards():
        raise S61_InnerCheckError("Slot 61 is for inner lamination only")

    if self.W1 <= self.W2:
        raise S61_W21CheckError("You must have W2 < W1")

    if self.H3 + self.H4 >= self.H2:
        raise S61_WindHError("You must have W3+W4 < H2")

    if self.W3 >= (self.W1 - self.W2) / 2:
        raise S61_WindWError("You must have W3 < (W1-W2)/2")



def comp_angle_opening(self):
    """Compute the average opening angle of the Slot

    Parameters
    ----------
    self : SlotW16
        A SlotW16 object

    Returns
    -------
    alpha: float
        Average opening angle of the slot [rad]

    """

    return self.W0



from numpy import ndarray


def as_dict(self, **kwargs):
    """
    Convert this object in a json serializable dict (can be use in __init__).
    Optional keyword input parameter is for internal use only
    and may prevent json serializability.
    """

    DataKeeper_dict = dict()
    DataKeeper_dict["name"] = self.name
    DataKeeper_dict["symbol"] = self.symbol
    DataKeeper_dict["unit"] = self.unit
    if self._keeper_str is not None:
        DataKeeper_dict["keeper"] = self._keeper_str
    elif "keep_function" in kwargs and kwargs["keep_function"]:
        DataKeeper_dict["keeper"] = self.keeper
    else:
        DataKeeper_dict["keeper"] = None
        if self.keeper is not None:
            self.get_logger().warning(
                "DataKeeper.as_dict(): "
                + f"Function {self.keeper.__name__} is not serializable "
                + "and will be converted to None."
            )
    if self._error_keeper_str is not None:
        DataKeeper_dict["error_keeper"] = self._error_keeper_str
    elif "keep_function" in kwargs and kwargs["keep_function"]:
        DataKeeper_dict["error_keeper"] = self.error_keeper
    else:
        DataKeeper_dict["error_keeper"] = None
        if self.error_keeper is not None:
            self.get_logger().warning(
                "DataKeeper.as_dict(): "
                + f"Function {self.error_keeper.__name__} is not serializable "
                + "and will be converted to None."
            )
    DataKeeper_dict["result"] = self.result.copy() if self.result is not None else None
    if DataKeeper_dict["result"] is not None:
        for ii in range(len(DataKeeper_dict["result"])):
            if hasattr(DataKeeper_dict["result"][ii], "as_dict"):
                DataKeeper_dict["result"][ii] = DataKeeper_dict["result"][ii].as_dict()
            elif isinstance(DataKeeper_dict["result"][ii], ndarray):
                DataKeeper_dict["result"][ii] = DataKeeper_dict["result"][ii].tolist()
    DataKeeper_dict["result_ref"] = self.result_ref
    if hasattr(DataKeeper_dict["result_ref"], "as_dict"):
        DataKeeper_dict["result_ref"] = DataKeeper_dict["result_ref"].as_dict()
    elif isinstance(DataKeeper_dict["result_ref"], ndarray):
        DataKeeper_dict["result_ref"] = DataKeeper_dict["result_ref"].tolist()
    # The class name is added to the dict for deserialisation purpose
    DataKeeper_dict["__class__"] = "DataKeeper"
    return DataKeeper_dict



# -*- coding: utf-8 -*-


def comp_surface_active(self):
    """Compute the active surface of the conductor

    Parameters
    ----------
    self : CondType11
        A CondType11 object

    Returns
    -------
    Sact: float
        Surface without insulation [m**2]

    """

    Sact = self.Hwire * self.Wwire * self.Nwppc_tan * self.Nwppc_rad

    return Sact



# -*- coding: utf-8 -*-


class ElmerSectionError(Exception):
    pass


# class to represent type 'File' in Elmer
class File(str):
    pass


# class to represent type 'Variable' in Elmer
# TODO multiple variable as list
class Variable:
    def __init__(self, name=None, value=None):
        self.name = name
        self.value = value


# class to represent MATC expressions in Elmer
class MATC:
    def __init__(self, expr=None):
        self.expr = expr



from ....Functions.Geometry.comp_flower_arc import comp_flower_arc
from ....Classes.Arc1 import Arc1
from ....Methods import ParentMissingError
from numpy import pi, exp


def get_bore_line(self, label=None):
    """Return the bore line description

    Parameters
    ----------
    self : BoreFlower
        A BoreFlower object

    Returns
    -------
    bore_list : list
        List of bore lines
    """

    if self.parent is not None:
        Rbo = self.parent.get_Rbo()
    else:
        raise ParentMissingError("Error: The Bore object is not inside a Lamination")

    if label is None:
        label = ""
    # Compute the shape
    (alpha_lim, z_top_left, z_top_right) = comp_flower_arc(
        2 * pi / self.N, self.Rarc, Rbo
    )

    # Create the lines
    bore_list = list()
    for ii in range(self.N):
        bore_list.append(
            Arc1(
                begin=z_top_right * exp(1j * (2 * pi / self.N * (ii - 1) + self.alpha)),
                end=z_top_left * exp(1j * (2 * pi / self.N * (ii - 1) + self.alpha)),
                radius=self.Rarc,
                is_trigo_direction=True,
                label=label,
            )
        )
    return bore_list



# -*- coding: utf-8 -*-

from numpy import arcsin, arctan, cos, exp, array, angle, pi
from numpy import imag as np_imag
from scipy.optimize import fsolve

from ....Classes.Segment import Segment
from ....Classes.SurfLine import SurfLine
from ....Classes.Arc1 import Arc1
from ....Methods import ParentMissingError


def build_geometry(self, alpha=0, delta=0, is_simplified=False):
    """Compute the curve (Segment) needed to plot the Hole.
    The ending point of a curve is the starting point of the next curve in
    the list

    Parameters
    ----------
    self : HoleUD
        A HoleUD object
    alpha : float
        Angle to rotate the slot (Default value = 0) [rad]
    delta : complex
        Complex to translate the slot (Default value = 0)
    is_simplified : bool
       True to avoid line superposition (not used)

    Returns
    -------
    surf_list: list
        List of SurfLine needed to draw the Hole
    """

    surf_list = self.surf_list

    try:
        if self.get_is_stator():
            st = "_Stator"
        else:
            st = "_Rotor"
    except ParentMissingError:
        st = "_None"

    # Update surface labels
    hole_id = 0
    mag_id = 0
    for surf in surf_list:
        if "HoleMagnet" in surf.label:
            key = "magnet_" + str(mag_id)
            if key in self.magnet_dict and self.magnet_dict[key] is not None:
                mag = self.magnet_dict[key]
                if mag.type_magnetization == 0:
                    type_mag = "_Radial"
                else:
                    type_mag = "_Parallel"
                surf.label = (
                    "HoleMagnet" + st + type_mag + "_N_R0_T" + str(mag_id) + "_S0"
                )
                mag_id += 1
            else:  # Magnet disabled or not defined
                surf.label = "Hole" + st + "_R0_T" + str(hole_id) + "_S0"
                hole_id += 1
        elif "Hole" in surf.label:
            surf.label = "Hole" + st + "_R0_T" + str(hole_id) + "_S0"
            hole_id += 1

    # Apply the transformations
    return_list = list()
    for surf in surf_list:
        return_list.append(surf.copy())
        return_list[-1].rotate(alpha)
        return_list[-1].translate(delta)

    return return_list



# -*- coding: utf-8 -*-
from ....definitions import PACKAGE_NAME

import numpy as np


def get_group(self, element):
    """Define a new PointMat object based on a set of elements.

    Parameters
    ----------
    self : PointMat
        an PointMat object
    element : Element
        an Element object

    Returns
    -------
    node: Node
        a Node object corresponding to Element

    """
    module = __import__(PACKAGE_NAME + ".Classes." + "PointMat", fromlist=["PointMat"])
    node = getattr(module, "PointMat")()

    node_tags = element.get_all_node_tags()

    node.nb_node = len(node_tags)
    node.coordinate = np.zeros((node.nb_node, 2))  # TO BE Extended to 3D
    node.tag = np.zeros((node.nb_node))
    for ind in range(node.nb_node):
        Ipos = np.where(node_tags[ind] == self.tag)[0]
        node.coordinate[ind, :] = self.coordinate[Ipos, :]
        node.tag[ind] = self.tag[Ipos]

    return node



# -*- coding: utf-8 -*-
from numpy import exp, pi

from ...Classes.SurfLine import SurfLine


def get_airgap_surface(lam_int, lam_ext):
    """Returns a list of surface in the airgap without sliding band surface

    Parameters
    ----------
    lam_int:
        Internal lamination
    lam_ext:
        External lamination

    Returns
    -------
    surf_list: list
        List of surface in the airgap including the sliding band surface
    """

    Rgap_mec_int = lam_int.comp_radius_mec()
    Rgap_mec_ext = lam_ext.comp_radius_mec()
    Wgap_mec = Rgap_mec_ext - Rgap_mec_int
    W_sb = Wgap_mec / 3  # Width sliding band
    surf_list = list()

    # Middle
    surf_list.append(
        SurfLine(
            line_list=[],
            point_ref=(Rgap_mec_int + W_sb * 3 / 2) * exp(1j * pi / 2),
            label="Airgap",
        )
    )

    return surf_list



# -*- coding: utf-8 -*-


def update(indiv):
    """Update the individual output after the mutation

    Parameters
    ----------
        indiv : Individual

    """

    indiv.output = type(indiv.output)(simu=indiv.output.simu.as_dict())

    for k, setter in enumerate(indiv.setter_list):
        setter(indiv.output.simu, indiv[k])

    indiv.is_simu_valid = False
    indiv.cstr_viol = 0

    # Delete the fitness
    del indiv.fitness.values



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Machine/Winding.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Machine/Winding
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from ._frozen import FrozenClass

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Machine.Winding.comp_Ncspc import comp_Ncspc
except ImportError as error:
    comp_Ncspc = error

try:
    from ..Methods.Machine.Winding.comp_Ntspc import comp_Ntspc
except ImportError as error:
    comp_Ntspc = error

try:
    from ..Methods.Machine.Winding.comp_phasor_angle import comp_phasor_angle
except ImportError as error:
    comp_phasor_angle = error

try:
    from ..Methods.Machine.Winding.comp_winding_factor import comp_winding_factor
except ImportError as error:
    comp_winding_factor = error

try:
    from ..Methods.Machine.Winding.comp_length_endwinding import comp_length_endwinding
except ImportError as error:
    comp_length_endwinding = error


from ._check import InitUnKnowClassError
from .Conductor import Conductor


class Winding(FrozenClass):
    """Winding abstract class"""

    VERSION = 1
    NAME = "Abstract Winding"

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Machine.Winding.comp_Ncspc
    if isinstance(comp_Ncspc, ImportError):
        comp_Ncspc = property(
            fget=lambda x: raise_(
                ImportError("Can't use Winding method comp_Ncspc: " + str(comp_Ncspc))
            )
        )
    else:
        comp_Ncspc = comp_Ncspc
    # cf Methods.Machine.Winding.comp_Ntspc
    if isinstance(comp_Ntspc, ImportError):
        comp_Ntspc = property(
            fget=lambda x: raise_(
                ImportError("Can't use Winding method comp_Ntspc: " + str(comp_Ntspc))
            )
        )
    else:
        comp_Ntspc = comp_Ntspc
    # cf Methods.Machine.Winding.comp_phasor_angle
    if isinstance(comp_phasor_angle, ImportError):
        comp_phasor_angle = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Winding method comp_phasor_angle: "
                    + str(comp_phasor_angle)
                )
            )
        )
    else:
        comp_phasor_angle = comp_phasor_angle
    # cf Methods.Machine.Winding.comp_winding_factor
    if isinstance(comp_winding_factor, ImportError):
        comp_winding_factor = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Winding method comp_winding_factor: "
                    + str(comp_winding_factor)
                )
            )
        )
    else:
        comp_winding_factor = comp_winding_factor
    # cf Methods.Machine.Winding.comp_length_endwinding
    if isinstance(comp_length_endwinding, ImportError):
        comp_length_endwinding = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Winding method comp_length_endwinding: "
                    + str(comp_length_endwinding)
                )
            )
        )
    else:
        comp_length_endwinding = comp_length_endwinding
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        is_reverse_wind=False,
        Nslot_shift_wind=0,
        qs=3,
        Ntcoil=7,
        Npcpp=2,
        type_connection=0,
        p=3,
        Lewout=0.015,
        conductor=-1,
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "is_reverse_wind" in list(init_dict.keys()):
                is_reverse_wind = init_dict["is_reverse_wind"]
            if "Nslot_shift_wind" in list(init_dict.keys()):
                Nslot_shift_wind = init_dict["Nslot_shift_wind"]
            if "qs" in list(init_dict.keys()):
                qs = init_dict["qs"]
            if "Ntcoil" in list(init_dict.keys()):
                Ntcoil = init_dict["Ntcoil"]
            if "Npcpp" in list(init_dict.keys()):
                Npcpp = init_dict["Npcpp"]
            if "type_connection" in list(init_dict.keys()):
                type_connection = init_dict["type_connection"]
            if "p" in list(init_dict.keys()):
                p = init_dict["p"]
            if "Lewout" in list(init_dict.keys()):
                Lewout = init_dict["Lewout"]
            if "conductor" in list(init_dict.keys()):
                conductor = init_dict["conductor"]
        # Set the properties (value check and convertion are done in setter)
        self.parent = None
        self.is_reverse_wind = is_reverse_wind
        self.Nslot_shift_wind = Nslot_shift_wind
        self.qs = qs
        self.Ntcoil = Ntcoil
        self.Npcpp = Npcpp
        self.type_connection = type_connection
        self.p = p
        self.Lewout = Lewout
        self.conductor = conductor

        # The class is frozen, for now it's impossible to add new properties
        self._freeze()

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        Winding_str = ""
        if self.parent is None:
            Winding_str += "parent = None " + linesep
        else:
            Winding_str += "parent = " + str(type(self.parent)) + " object" + linesep
        Winding_str += "is_reverse_wind = " + str(self.is_reverse_wind) + linesep
        Winding_str += "Nslot_shift_wind = " + str(self.Nslot_shift_wind) + linesep
        Winding_str += "qs = " + str(self.qs) + linesep
        Winding_str += "Ntcoil = " + str(self.Ntcoil) + linesep
        Winding_str += "Npcpp = " + str(self.Npcpp) + linesep
        Winding_str += "type_connection = " + str(self.type_connection) + linesep
        Winding_str += "p = " + str(self.p) + linesep
        Winding_str += "Lewout = " + str(self.Lewout) + linesep
        if self.conductor is not None:
            tmp = self.conductor.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            Winding_str += "conductor = " + tmp
        else:
            Winding_str += "conductor = None" + linesep + linesep
        return Winding_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False
        if other.is_reverse_wind != self.is_reverse_wind:
            return False
        if other.Nslot_shift_wind != self.Nslot_shift_wind:
            return False
        if other.qs != self.qs:
            return False
        if other.Ntcoil != self.Ntcoil:
            return False
        if other.Npcpp != self.Npcpp:
            return False
        if other.type_connection != self.type_connection:
            return False
        if other.p != self.p:
            return False
        if other.Lewout != self.Lewout:
            return False
        if other.conductor != self.conductor:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()
        if other._is_reverse_wind != self._is_reverse_wind:
            diff_list.append(name + ".is_reverse_wind")
        if other._Nslot_shift_wind != self._Nslot_shift_wind:
            diff_list.append(name + ".Nslot_shift_wind")
        if other._qs != self._qs:
            diff_list.append(name + ".qs")
        if other._Ntcoil != self._Ntcoil:
            diff_list.append(name + ".Ntcoil")
        if other._Npcpp != self._Npcpp:
            diff_list.append(name + ".Npcpp")
        if other._type_connection != self._type_connection:
            diff_list.append(name + ".type_connection")
        if other._p != self._p:
            diff_list.append(name + ".p")
        if other._Lewout != self._Lewout:
            diff_list.append(name + ".Lewout")
        if (other.conductor is None and self.conductor is not None) or (
            other.conductor is not None and self.conductor is None
        ):
            diff_list.append(name + ".conductor None mismatch")
        elif self.conductor is not None:
            diff_list.extend(
                self.conductor.compare(other.conductor, name=name + ".conductor")
            )
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object
        S += getsizeof(self.is_reverse_wind)
        S += getsizeof(self.Nslot_shift_wind)
        S += getsizeof(self.qs)
        S += getsizeof(self.Ntcoil)
        S += getsizeof(self.Npcpp)
        S += getsizeof(self.type_connection)
        S += getsizeof(self.p)
        S += getsizeof(self.Lewout)
        S += getsizeof(self.conductor)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        Winding_dict = dict()
        Winding_dict["is_reverse_wind"] = self.is_reverse_wind
        Winding_dict["Nslot_shift_wind"] = self.Nslot_shift_wind
        Winding_dict["qs"] = self.qs
        Winding_dict["Ntcoil"] = self.Ntcoil
        Winding_dict["Npcpp"] = self.Npcpp
        Winding_dict["type_connection"] = self.type_connection
        Winding_dict["p"] = self.p
        Winding_dict["Lewout"] = self.Lewout
        if self.conductor is None:
            Winding_dict["conductor"] = None
        else:
            Winding_dict["conductor"] = self.conductor.as_dict(**kwargs)
        # The class name is added to the dict for deserialisation purpose
        Winding_dict["__class__"] = "Winding"
        return Winding_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.is_reverse_wind = None
        self.Nslot_shift_wind = None
        self.qs = None
        self.Ntcoil = None
        self.Npcpp = None
        self.type_connection = None
        self.p = None
        self.Lewout = None
        if self.conductor is not None:
            self.conductor._set_None()

    def _get_is_reverse_wind(self):
        """getter of is_reverse_wind"""
        return self._is_reverse_wind

    def _set_is_reverse_wind(self, value):
        """setter of is_reverse_wind"""
        check_var("is_reverse_wind", value, "bool")
        self._is_reverse_wind = value

    is_reverse_wind = property(
        fget=_get_is_reverse_wind,
        fset=_set_is_reverse_wind,
        doc=u"""1 to reverse the default winding algorithm along the airgap (c, b, a instead of a, b, c along the trigonometric direction)

        :Type: bool
        """,
    )

    def _get_Nslot_shift_wind(self):
        """getter of Nslot_shift_wind"""
        return self._Nslot_shift_wind

    def _set_Nslot_shift_wind(self, value):
        """setter of Nslot_shift_wind"""
        check_var("Nslot_shift_wind", value, "int")
        self._Nslot_shift_wind = value

    Nslot_shift_wind = property(
        fget=_get_Nslot_shift_wind,
        fset=_set_Nslot_shift_wind,
        doc=u"""0 not to change the stator winding connection matrix built by pyleecan number of slots to shift the coils obtained with pyleecan winding algorithm (a, b, c becomes b, c, a with Nslot_shift_wind1=1)

        :Type: int
        """,
    )

    def _get_qs(self):
        """getter of qs"""
        return self._qs

    def _set_qs(self, value):
        """setter of qs"""
        check_var("qs", value, "int", Vmin=0, Vmax=100)
        self._qs = value

    qs = property(
        fget=_get_qs,
        fset=_set_qs,
        doc=u"""number of phases 

        :Type: int
        :min: 0
        :max: 100
        """,
    )

    def _get_Ntcoil(self):
        """getter of Ntcoil"""
        return self._Ntcoil

    def _set_Ntcoil(self, value):
        """setter of Ntcoil"""
        check_var("Ntcoil", value, "int", Vmin=1, Vmax=1000)
        self._Ntcoil = value

    Ntcoil = property(
        fget=_get_Ntcoil,
        fset=_set_Ntcoil,
        doc=u"""number of turns per coil

        :Type: int
        :min: 1
        :max: 1000
        """,
    )

    def _get_Npcpp(self):
        """getter of Npcpp"""
        return self._Npcpp

    def _set_Npcpp(self, value):
        """setter of Npcpp"""
        check_var("Npcpp", value, "int", Vmin=1, Vmax=1000)
        self._Npcpp = value

    Npcpp = property(
        fget=_get_Npcpp,
        fset=_set_Npcpp,
        doc=u"""number of parallel circuits per phase (maximum 2p)

        :Type: int
        :min: 1
        :max: 1000
        """,
    )

    def _get_type_connection(self):
        """getter of type_connection"""
        return self._type_connection

    def _set_type_connection(self, value):
        """setter of type_connection"""
        check_var("type_connection", value, "int", Vmin=0, Vmax=1)
        self._type_connection = value

    type_connection = property(
        fget=_get_type_connection,
        fset=_set_type_connection,
        doc=u"""Winding connection : 0 star (Y), 1 triangle (delta)

        :Type: int
        :min: 0
        :max: 1
        """,
    )

    def _get_p(self):
        """getter of p"""
        return self._p

    def _set_p(self, value):
        """setter of p"""
        check_var("p", value, "int", Vmin=1, Vmax=100)
        self._p = value

    p = property(
        fget=_get_p,
        fset=_set_p,
        doc=u"""pole pairs number

        :Type: int
        :min: 1
        :max: 100
        """,
    )

    def _get_Lewout(self):
        """getter of Lewout"""
        return self._Lewout

    def _set_Lewout(self, value):
        """setter of Lewout"""
        check_var("Lewout", value, "float", Vmin=0, Vmax=100)
        self._Lewout = value

    Lewout = property(
        fget=_get_Lewout,
        fset=_set_Lewout,
        doc=u"""straight length of the conductors outside the lamination before the curved part of winding overhang [m] - can be negative to tune the average turn length 

        :Type: float
        :min: 0
        :max: 100
        """,
    )

    def _get_conductor(self):
        """getter of conductor"""
        return self._conductor

    def _set_conductor(self, value):
        """setter of conductor"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "pyleecan.Classes", value.get("__class__"), "conductor"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = Conductor()
        check_var("conductor", value, "Conductor")
        self._conductor = value

        if self._conductor is not None:
            self._conductor.parent = self

    conductor = property(
        fget=_get_conductor,
        fset=_set_conductor,
        doc=u"""Winding's conductor

        :Type: Conductor
        """,
    )



# -*- coding: utf-8 -*-
"""File generated according to DMatSetup/gen_list.json
WARNING! All changes made in this file will be lost!
"""
from pyleecan.GUI.Dialog.DMatLib.DMatSetup.Ui_DMatSetup import Ui_DMatSetup


class Gen_DMatSetup(Ui_DMatSetup):
    def setupUi(self, DMatSetup):
        """Abstract class to update the widget according to the csv doc"""
        Ui_DMatSetup.setupUi(self, DMatSetup)
        # Setup of in_name
        txt = self.tr(u"""name of the material""")
        self.in_name.setWhatsThis(txt)
        self.in_name.setToolTip(txt)

        # Setup of le_name
        txt = self.tr(u"""name of the material""")
        self.le_name.setWhatsThis(txt)
        self.le_name.setToolTip(txt)

        # Setup of is_isotropic
        txt = self.tr(u"""If True, uniformity in all orientations""")
        self.is_isotropic.setWhatsThis(txt)
        self.is_isotropic.setToolTip(txt)

        # Setup of in_rho_elec
        txt = self.tr(u"""Resistivity at 20 deg C""")
        self.in_rho_elec.setWhatsThis(txt)
        self.in_rho_elec.setToolTip(txt)

        # Setup of lf_rho_elec
        self.lf_rho_elec.validator().setBottom(0)
        txt = self.tr(u"""Resistivity at 20 deg C""")
        self.lf_rho_elec.setWhatsThis(txt)
        self.lf_rho_elec.setToolTip(txt)

        # Setup of unit_rho_elec
        txt = self.tr(u"""Resistivity at 20 deg C""")
        self.unit_rho_elec.setWhatsThis(txt)
        self.unit_rho_elec.setToolTip(txt)

        # Setup of in_cost_unit
        txt = self.tr(u"""Cost of one kilo of material""")
        self.in_cost_unit.setWhatsThis(txt)
        self.in_cost_unit.setToolTip(txt)

        # Setup of lf_cost_unit
        self.lf_cost_unit.validator().setBottom(0)
        txt = self.tr(u"""Cost of one kilo of material""")
        self.lf_cost_unit.setWhatsThis(txt)
        self.lf_cost_unit.setToolTip(txt)

        # Setup of unit_cost_unit
        txt = self.tr(u"""Cost of one kilo of material""")
        self.unit_cost_unit.setWhatsThis(txt)
        self.unit_cost_unit.setToolTip(txt)

        # Setup of in_Cp
        txt = self.tr(u"""specific heat capacity""")
        self.in_Cp.setWhatsThis(txt)
        self.in_Cp.setToolTip(txt)

        # Setup of lf_Cp
        self.lf_Cp.validator().setBottom(0)
        txt = self.tr(u"""specific heat capacity""")
        self.lf_Cp.setWhatsThis(txt)
        self.lf_Cp.setToolTip(txt)

        # Setup of unit_Cp
        txt = self.tr(u"""specific heat capacity""")
        self.unit_Cp.setWhatsThis(txt)
        self.unit_Cp.setToolTip(txt)

        # Setup of in_alpha
        txt = self.tr(u"""thermal expansion coefficient""")
        self.in_alpha.setWhatsThis(txt)
        self.in_alpha.setToolTip(txt)

        # Setup of lf_alpha
        self.lf_alpha.validator().setBottom(0)
        txt = self.tr(u"""thermal expansion coefficient""")
        self.lf_alpha.setWhatsThis(txt)
        self.lf_alpha.setToolTip(txt)

        # Setup of in_L
        txt = self.tr(
            u"""thermal conductivity (XY is lamination plane, Z is rotation axis)"""
        )
        self.in_L.setWhatsThis(txt)
        self.in_L.setToolTip(txt)

        # Setup of unit_L
        txt = self.tr(
            u"""thermal conductivity (XY is lamination plane, Z is rotation axis)"""
        )
        self.unit_L.setWhatsThis(txt)
        self.unit_L.setToolTip(txt)

        # Setup of lf_L
        self.lf_L.validator().setBottom(0)
        txt = self.tr(
            u"""thermal conductivity (XY is lamination plane, Z is rotation axis)"""
        )
        self.lf_L.setWhatsThis(txt)
        self.lf_L.setToolTip(txt)

        # Setup of in_Lx
        txt = self.tr(
            u"""thermal conductivity (XY is lamination plane, Z is rotation axis)"""
        )
        self.in_Lx.setWhatsThis(txt)
        self.in_Lx.setToolTip(txt)

        # Setup of lf_Lx
        self.lf_Lx.validator().setBottom(0)
        txt = self.tr(
            u"""thermal conductivity (XY is lamination plane, Z is rotation axis)"""
        )
        self.lf_Lx.setWhatsThis(txt)
        self.lf_Lx.setToolTip(txt)

        # Setup of in_Ly
        txt = self.tr(
            u"""thermal conductivity (XY is lamination plane, Z is rotation axis)"""
        )
        self.in_Ly.setWhatsThis(txt)
        self.in_Ly.setToolTip(txt)

        # Setup of lf_Ly
        self.lf_Ly.validator().setBottom(0)
        txt = self.tr(
            u"""thermal conductivity (XY is lamination plane, Z is rotation axis)"""
        )
        self.lf_Ly.setWhatsThis(txt)
        self.lf_Ly.setToolTip(txt)

        # Setup of in_Lz
        txt = self.tr(
            u"""thermal conductivity (XY is lamination plane, Z is rotation axis)"""
        )
        self.in_Lz.setWhatsThis(txt)
        self.in_Lz.setToolTip(txt)

        # Setup of lf_Lz
        self.lf_Lz.validator().setBottom(0)
        txt = self.tr(
            u"""thermal conductivity (XY is lamination plane, Z is rotation axis)"""
        )
        self.lf_Lz.setWhatsThis(txt)
        self.lf_Lz.setToolTip(txt)

        # Setup of in_rho_meca
        txt = self.tr(u"""mass per unit volume [kg/m3]""")
        self.in_rho_meca.setWhatsThis(txt)
        self.in_rho_meca.setToolTip(txt)

        # Setup of lf_rho_meca
        self.lf_rho_meca.validator().setBottom(0)
        txt = self.tr(u"""mass per unit volume [kg/m3]""")
        self.lf_rho_meca.setWhatsThis(txt)
        self.lf_rho_meca.setToolTip(txt)

        # Setup of unit_rho_meca
        txt = self.tr(u"""mass per unit volume [kg/m3]""")
        self.unit_rho_meca.setWhatsThis(txt)
        self.unit_rho_meca.setToolTip(txt)

        # Setup of in_E
        txt = self.tr(
            u"""equivalent Young modulus (XY is lamination plane, Z is rotation axis)"""
        )
        self.in_E.setWhatsThis(txt)
        self.in_E.setToolTip(txt)

        # Setup of lf_E
        self.lf_E.validator().setBottom(0)
        txt = self.tr(
            u"""equivalent Young modulus (XY is lamination plane, Z is rotation axis)"""
        )
        self.lf_E.setWhatsThis(txt)
        self.lf_E.setToolTip(txt)

        # Setup of unit_E
        txt = self.tr(
            u"""equivalent Young modulus (XY is lamination plane, Z is rotation axis)"""
        )
        self.unit_E.setWhatsThis(txt)
        self.unit_E.setToolTip(txt)

        # Setup of in_Ex
        txt = self.tr(
            u"""equivalent Young modulus (XY is lamination plane, Z is rotation axis)"""
        )
        self.in_Ex.setWhatsThis(txt)
        self.in_Ex.setToolTip(txt)

        # Setup of lf_Ex
        self.lf_Ex.validator().setBottom(0)
        txt = self.tr(
            u"""equivalent Young modulus (XY is lamination plane, Z is rotation axis)"""
        )
        self.lf_Ex.setWhatsThis(txt)
        self.lf_Ex.setToolTip(txt)

        # Setup of in_Ey
        txt = self.tr(
            u"""equivalent Young modulus (XY is lamination plane, Z is rotation axis)"""
        )
        self.in_Ey.setWhatsThis(txt)
        self.in_Ey.setToolTip(txt)

        # Setup of lf_Ey
        self.lf_Ey.validator().setBottom(0)
        txt = self.tr(
            u"""equivalent Young modulus (XY is lamination plane, Z is rotation axis)"""
        )
        self.lf_Ey.setWhatsThis(txt)
        self.lf_Ey.setToolTip(txt)

        # Setup of in_Ez
        txt = self.tr(
            u"""equivalent Young modulus (XY is lamination plane, Z is rotation axis)"""
        )
        self.in_Ez.setWhatsThis(txt)
        self.in_Ez.setToolTip(txt)

        # Setup of lf_Ez
        self.lf_Ez.validator().setBottom(0)
        txt = self.tr(
            u"""equivalent Young modulus (XY is lamination plane, Z is rotation axis)"""
        )
        self.lf_Ez.setWhatsThis(txt)
        self.lf_Ez.setToolTip(txt)

        # Setup of in_G
        txt = self.tr(
            u"""shear modulus in XZ plane (XY is lamination plane, Z is rotation axis)"""
        )
        self.in_G.setWhatsThis(txt)
        self.in_G.setToolTip(txt)

        # Setup of lf_G
        self.lf_G.validator().setBottom(0)
        txt = self.tr(
            u"""shear modulus in XZ plane (XY is lamination plane, Z is rotation axis)"""
        )
        self.lf_G.setWhatsThis(txt)
        self.lf_G.setToolTip(txt)

        # Setup of unit_G
        txt = self.tr(
            u"""shear modulus in XZ plane (XY is lamination plane, Z is rotation axis)"""
        )
        self.unit_G.setWhatsThis(txt)
        self.unit_G.setToolTip(txt)

        # Setup of in_Gxy
        txt = self.tr(
            u"""shear modulus in XZ plane (XY is lamination plane, Z is rotation axis)"""
        )
        self.in_Gxy.setWhatsThis(txt)
        self.in_Gxy.setToolTip(txt)

        # Setup of lf_Gxy
        self.lf_Gxy.validator().setBottom(0)
        txt = self.tr(
            u"""shear modulus in XZ plane (XY is lamination plane, Z is rotation axis)"""
        )
        self.lf_Gxy.setWhatsThis(txt)
        self.lf_Gxy.setToolTip(txt)

        # Setup of in_Gxz
        txt = self.tr(
            u"""shear modulus in XY plane (XY is lamination plane, Z is rotation axis)"""
        )
        self.in_Gxz.setWhatsThis(txt)
        self.in_Gxz.setToolTip(txt)

        # Setup of lf_Gxz
        self.lf_Gxz.validator().setBottom(0)
        txt = self.tr(
            u"""shear modulus in XY plane (XY is lamination plane, Z is rotation axis)"""
        )
        self.lf_Gxz.setWhatsThis(txt)
        self.lf_Gxz.setToolTip(txt)

        # Setup of in_Gyz
        txt = self.tr(
            u"""shear modulus in YZ plane (XY is lamination plane, Z is rotation axis)"""
        )
        self.in_Gyz.setWhatsThis(txt)
        self.in_Gyz.setToolTip(txt)

        # Setup of lf_Gyz
        self.lf_Gyz.validator().setBottom(0)
        txt = self.tr(
            u"""shear modulus in YZ plane (XY is lamination plane, Z is rotation axis)"""
        )
        self.lf_Gyz.setWhatsThis(txt)
        self.lf_Gyz.setToolTip(txt)

        # Setup of in_nu
        txt = self.tr(
            u"""equivalent Poisson ratio in the XY plane (XY is lamination plane, Z is rotation axis)"""
        )
        self.in_nu.setWhatsThis(txt)
        self.in_nu.setToolTip(txt)

        # Setup of lf_nu
        self.lf_nu.validator().setBottom(0)
        txt = self.tr(
            u"""equivalent Poisson ratio in the XY plane (XY is lamination plane, Z is rotation axis)"""
        )
        self.lf_nu.setWhatsThis(txt)
        self.lf_nu.setToolTip(txt)

        # Setup of in_nu_xy
        txt = self.tr(
            u"""equivalent Poisson ratio in the XY plane (XY is lamination plane, Z is rotation axis)"""
        )
        self.in_nu_xy.setWhatsThis(txt)
        self.in_nu_xy.setToolTip(txt)

        # Setup of lf_nu_xy
        self.lf_nu_xy.validator().setBottom(0)
        txt = self.tr(
            u"""equivalent Poisson ratio in the XY plane (XY is lamination plane, Z is rotation axis)"""
        )
        self.lf_nu_xy.setWhatsThis(txt)
        self.lf_nu_xy.setToolTip(txt)

        # Setup of in_nu_xz
        txt = self.tr(
            u"""equivalent Poisson ratio in the XZ plane (XY is lamination plane, Z is rotation axis)"""
        )
        self.in_nu_xz.setWhatsThis(txt)
        self.in_nu_xz.setToolTip(txt)

        # Setup of lf_nu_xz
        self.lf_nu_xz.validator().setBottom(0)
        txt = self.tr(
            u"""equivalent Poisson ratio in the XZ plane (XY is lamination plane, Z is rotation axis)"""
        )
        self.lf_nu_xz.setWhatsThis(txt)
        self.lf_nu_xz.setToolTip(txt)

        # Setup of in_nu_yz
        txt = self.tr(
            u"""equivalent Poisson ratio in the YZ plane (XY is lamination plane, Z is rotation axis)"""
        )
        self.in_nu_yz.setWhatsThis(txt)
        self.in_nu_yz.setToolTip(txt)

        # Setup of lf_nu_yz
        self.lf_nu_yz.validator().setBottom(0)
        txt = self.tr(
            u"""equivalent Poisson ratio in the YZ plane (XY is lamination plane, Z is rotation axis)"""
        )
        self.lf_nu_yz.setWhatsThis(txt)
        self.lf_nu_yz.setToolTip(txt)

        # Setup of in_mur_lin
        txt = self.tr(u"""Relative magnetic permeability""")
        self.in_mur_lin.setWhatsThis(txt)
        self.in_mur_lin.setToolTip(txt)

        # Setup of lf_mur_lin
        self.lf_mur_lin.validator().setBottom(0)
        txt = self.tr(u"""Relative magnetic permeability""")
        self.lf_mur_lin.setWhatsThis(txt)
        self.lf_mur_lin.setToolTip(txt)

        # Setup of in_Brm20
        txt = self.tr(u"""magnet remanence induction at 20degC""")
        self.in_Brm20.setWhatsThis(txt)
        self.in_Brm20.setToolTip(txt)

        # Setup of lf_Brm20
        txt = self.tr(u"""magnet remanence induction at 20degC""")
        self.lf_Brm20.setWhatsThis(txt)
        self.lf_Brm20.setToolTip(txt)

        # Setup of unit_Brm20
        txt = self.tr(u"""magnet remanence induction at 20degC""")
        self.unit_Brm20.setWhatsThis(txt)
        self.unit_Brm20.setToolTip(txt)

        # Setup of in_alpha_Br
        txt = self.tr(
            u"""temperature coefficient for remanent flux density /degC compared to 20degC"""
        )
        self.in_alpha_Br.setWhatsThis(txt)
        self.in_alpha_Br.setToolTip(txt)

        # Setup of lf_alpha_Br
        txt = self.tr(
            u"""temperature coefficient for remanent flux density /degC compared to 20degC"""
        )
        self.lf_alpha_Br.setWhatsThis(txt)
        self.lf_alpha_Br.setToolTip(txt)

        # Setup of in_Wlam
        txt = self.tr(
            u"""lamination sheet width without insulation [m] (0 == not laminated)"""
        )
        self.in_Wlam.setWhatsThis(txt)
        self.in_Wlam.setToolTip(txt)

        # Setup of lf_Wlam
        self.lf_Wlam.validator().setBottom(0)
        txt = self.tr(
            u"""lamination sheet width without insulation [m] (0 == not laminated)"""
        )
        self.lf_Wlam.setWhatsThis(txt)
        self.lf_Wlam.setToolTip(txt)

        # Setup of unit_Wlam
        txt = self.tr(
            u"""lamination sheet width without insulation [m] (0 == not laminated)"""
        )
        self.unit_Wlam.setWhatsThis(txt)
        self.unit_Wlam.setToolTip(txt)



from os.path import isfile
from importlib import import_module
from ....Classes._check import CheckTypeError
from ntpath import basename


def _set_setter(self, value):
    """setter of setter"""
    # Create the function if setter is defined as a string (e.g. simu.machine.rotor.slot.W0)
    if isinstance(value, str) and value[:4] == "simu":
        value_split = value.split(".")
        value = (
            "lambda simu, val: setattr(eval('"
            + ".".join(value_split[:-1])
            + "'), '"
            + value_split[-1]
            + "', val"
            + ")"
        )
    if value is None:
        self._setter_str = None
        self._setter_func = None
    elif isinstance(value, str) and "lambda" in value:
        self._setter_str = value
        self._setter_func = eval(value)
    elif isinstance(value, str) and isfile(value) and value[-3:] == ".py":
        self._setter_str = value
        f = open(value, "r")
        exec(f.read(), globals())
        self._setter_func = eval(basename(value[:-3]))
    elif callable(value):
        self._setter_str = None
        self._setter_func = value
    else:
        raise CheckTypeError(
            "For property keeper Expected function or str (path to python file or lambda), got: "
            + str(type(value))
        )



from ....Methods.Machine import MachineCheckError


class LaminationCheckError(MachineCheckError):
    """ """

    pass


class LC_RadiusError(LaminationCheckError):
    """  """

    pass


class LC_VentYokeOut(LaminationCheckError):
    """ """

    pass


class LC_VentYokeIn(LaminationCheckError):
    """ """

    pass



def __setitem__(self, key, value):
    """Called to implement assignment to self[key]."""
    if isinstance(key, int) or isinstance(key, slice):
        return self.output_list.__setitem__(key, value)
    else:
        return self.xoutput_dict.__setitem__(key, value)



# -*- coding: utf-8 -*-

import pyvista as pv
from numpy import (
    real,
    linspace,
    exp,
    pi,
    argmax,
    sum as np_sum,
    min as np_min,
    max as np_max,
)

from ....Classes.MeshMat import MeshMat
from ....Classes.MeshVTK import MeshVTK
from ....definitions import config_dict

COLOR_MAP = config_dict["PLOT"]["COLOR_DICT"]["COLOR_MAP"]


def plot_deflection_animated(
    self,
    label=None,
    index=None,
    indices=None,
    clim=None,
    factor=None,
    field_name=None,
    is_time=False,
    ifreq=None,
    gif_name="animation.gif",
    gif_path="./",
    title="",
):
    """Create the gif of the animated operational deflection shape.

    Parameters
    ----------
    self : MeshSolution
        a MeshSolution object
    label : str
        a label
    index : int
        an index
    indices : list
        list of the points to extract (optional)
    clim : list
        a list of 2 elements for the limits of the colorbar
    factor : float
        factor to multiply vector field
    field_name : str
        title of the field to display on plot
    gif_name : str
        name of the gif file (should end with .gif)
    gif_path : str
        path where gif will be saved

    Returns
    -------
    """

    # Get the mesh
    mesh = self.get_mesh(label=label, index=index)
    if isinstance(mesh, MeshMat):
        mesh_pv = mesh.get_mesh_pv(indices=indices)
        mesh = MeshVTK(mesh=mesh_pv, is_pyvista_mesh=True)

    # Get the field
    field = self.get_field(label=label, index=index, indices=indices, is_radial=True)
    vect_field = self.get_field(label=label, index=index, indices=indices)
    if is_time:
        field_data = real(field[:, 0])
        vect_field_data = real(vect_field[:, :, 0])
    elif len(field.shape) == 2:
        if ifreq is None:
            # Find frequency with highest response
            ifreq = argmax(np_sum(real(field), axis=0))
        field = field[:, ifreq]
        vect_field = vect_field[:, :, ifreq]
        field_data = real(field)
        vect_field_data = real(vect_field)

    else:
        field_data = real(field)
        vect_field_data = real(vect_field)
    if field_name is None:
        if label is not None:
            field_name = label
        elif self.get_solution(index=index).label is not None:
            field_name = self.get_solution(index=index).label
        else:
            field_name = "Field"

    # Compute colorbar boundaries
    if clim is None:
        clim = [np_min(real(field)), np_max(real(field))]
        if (clim[1] - clim[0]) / clim[1] < 0.01:
            clim[0] = -abs(clim[1])
            clim[1] = abs(clim[1])

    # Compute deformation factor
    if factor is None:
        factor = 1 / (100 * clim[1])

    # Extract surface
    surf = mesh.get_surf(indices=indices)

    # Add field to surf
    surf.vectors = real(vect_field_data) * factor

    # Warp by vectors
    surf_warp = surf.warp_by_vector()

    # Add radial field
    surf_warp[field_name] = real(field_data)

    # Configure plot
    pv.set_plot_theme("document")
    p = pv.Plotter(notebook=False, title=title)
    sargs = dict(
        interactive=True,
        title_font_size=20,
        label_font_size=16,
        font_family="arial",
        color="black",
    )
    p.add_mesh(
        surf_warp,
        scalars=field_name,
        opacity=1,
        show_edges=False,
        cmap=COLOR_MAP,
        clim=clim,
        scalar_bar_args=sargs,
    )
    p.add_text(title, position="upper_edge")
    p.show(use_panel=False, auto_close=False)

    # GIF
    if is_time:
        p.open_gif(gif_path + "/" + gif_name)
        for tind in range(field.shape[2]):
            field_data = real(field[:, tind])
            vect_field_data = real(vect_field[:, :, tind])
            surf.vectors = vect_field_data * factor
            surf_warp = surf.warp_by_vector()
            surf_warp[field_name] = field_data
            p.add_mesh(
                surf_warp,
                scalars=field_name,
                show_edges=False,
                cmap=COLOR_MAP,
                clim=clim,
                scalar_bar_args=sargs,
            )
            p.add_text(title, position="upper_edge")
            p.write_frame()
            p.clear()

    else:
        nframe = 25
        p.open_gif(gif_path + "/" + gif_name)
        for t in linspace(0.0, 1.0, nframe + 1)[:nframe]:
            vect_field_data = real(vect_field * exp(1j * 2 * pi * t))
            field_data = real(field * exp(1j * 2 * pi * t))
            surf.vectors = vect_field_data * factor
            surf_warp = surf.warp_by_vector()
            surf_warp[field_name] = field_data
            p.add_mesh(
                surf_warp,
                scalars=field_name,
                show_edges=False,
                cmap=COLOR_MAP,
                clim=clim,
                scalar_bar_args=sargs,
            )
            p.add_text(title, position="upper_edge")
            p.write_frame()
            p.clear()

    # Close movie and delete object
    p.close()



# -*- coding: utf-8 -*-
from ....Classes.Circle import Circle
from ....Classes.Arc1 import Arc1
from ....Classes.Segment import Segment
from ....Classes.SurfLine import SurfLine
from ....Classes.SurfRing import SurfRing
from numpy import exp, pi


def build_geometry(self, sym=1, alpha=0, delta=0):
    """Build the geometry of the Frame

    Parameters
    ----------
    self : Frame
        Frame Object
    sym : int
        symmetry factor (1= full machine, 2= half of the machine...)
    alpha : float
        Angle for rotation [rad]
    delta : complex
        Complex value for translation

    Returns
    -------
    surf_list : list
        list of surface

    """
    surf_list = list()
    # If there is a frame...
    if self.comp_height_eq() != 0:
        if sym == 1:  # No symmetry / full frame
            out_surf = Circle(
                radius=self.Rext,
                label="Frame_Outter",
                center=0,
                point_ref=self.Rint + (self.Rext - self.Rint) / 2,
            )
            in_surf = Circle(
                radius=self.Rint, label="Frame_Inner", center=0, point_ref=0
            )
            surf_list = [
                SurfRing(
                    out_surf=out_surf,
                    in_surf=in_surf,
                    label="Frame",
                    point_ref=self.Rint + (self.Rext - self.Rint) / 2,
                )
            ]
        else:  # Part of the frame
            Z0 = self.Rint
            Z3 = Z0 * exp(1j * 2 * pi / sym)
            Z1 = self.Rext
            Z2 = Z1 * exp(1j * 2 * pi / sym)
            curve_list = list()
            curve_list.append(Segment(Z0, Z1))
            curve_list.append(Arc1(Z1, Z2, self.Rext))
            curve_list.append(Segment(Z2, Z3))
            curve_list.append(Arc1(Z3, Z0, -self.Rint))
            surf_frame = SurfLine(
                line_list=curve_list,
                label="Frame",
                point_ref=self.Rint + (self.Rext - self.Rint) / 2,
            )
            surf_list.append(surf_frame)

        # Apply the transformations
        for surf in surf_list:
            surf.rotate(alpha)
            surf.translate(delta)

    return surf_list



from os import getcwd, rename, remove
from os.path import join, dirname, abspath, split
from re import match

from PySide2.QtWidgets import QDialog, QFileDialog, QMessageBox

from ....Functions.load import load_matlib
from ....Functions.Material.compare_material import compare_material
from ....Functions.Material.replace_material_pyleecan_obj import (
    replace_material_pyleecan_obj,
)
from ....GUI.Tools.GuiOption.Ui_GuiOption import Ui_GUIOption
from ....GUI.Dialog.DMatLib.DMatSetup.DMatSetup import DMatSetup
from ....GUI import gui_option
from ....Functions.path_tools import abs_file_path
from ....Functions.init_environment import save_config_dict
from ....definitions import config_dict
from ....Functions.load import load_matlib
from ....Classes.Material import Material


class WGuiOption(Ui_GUIOption, QDialog):
    def __init__(self, machine_setup=None, matlib=None):
        """
        WGuiOption enable to modify some option in the GUI such as:
            - units
            - material library folder

        Parameters:
        machine_setup: DMachineSetup
            Machine widget
        matlib : MatLib
            Material Library
        """
        QDialog.__init__(self)
        self.setupUi(self)
        # Matlib path selector setup
        self.w_matlib_path.verbose_name = "Matlib directory"
        self.w_matlib_path.is_file = False
        self.w_matlib_path.set_path_txt(config_dict["MAIN"]["MATLIB_DIR"])
        self.w_matlib_path.update()

        self.machine_setup = machine_setup  # DMachineSetup to access to the machine
        self.matlib = matlib  # dmatlib to access to the material library

        self.c_unit_m.setCurrentIndex(gui_option.unit.unit_m)
        self.c_unit_m2.setCurrentIndex(gui_option.unit.unit_m2)

        # Connections
        self.w_matlib_path.pathChanged.connect(self.change_matlib_dir)
        self.c_unit_m.currentTextChanged.connect(self.set_unit_m)
        self.c_unit_m2.currentTextChanged.connect(self.set_unit_m2)

    def change_matlib_dir(self):
        """
        Change the matlib directory and load the new matlib
        """
        matlib_path = self.w_matlib_path.get_path()
        config_dict["MAIN"]["MATLIB_DIR"] = matlib_path
        save_config_dict(config_dict)

        if self.matlib is not None:
            self.matlib.load_mat_ref(matlib_path)
            if self.machine_setup is not None:
                self.matlib.add_machine_mat(self.machine_setup.machine)

    def set_unit_m(self):
        """Update the value of unit_m"""
        global gui_option
        gui_option.unit.unit_m = self.c_unit_m.currentIndex()
        config_dict["GUI"]["UNIT_M"] = gui_option.unit.unit_m
        save_config_dict(config_dict)

    def set_unit_m2(self):
        """Update the value of unit_m2"""
        global gui_option
        gui_option.unit.unit_m2 = self.c_unit_m2.currentIndex()
        config_dict["GUI"]["UNIT_M2"] = gui_option.unit.unit_m2
        save_config_dict(config_dict)



# -*- coding: utf-8 -*-

from os.path import join, abspath, isdir
from os import makedirs
from .....definitions import RESULT_DIR


def get_path_result(self):
    """Return the path to the result folder. If there is a name for the simulation, the path leads to a sub-folder of
    'Results' folder which has the same name. If this sub-folder does not exist, it creates it.

    Parameters
    ----------
    self : Output
        an Output object

    Returns
    -------
    res_path: str
        path to the result folder
    """

    if self.path_result in [None, ""]:
        if self.simu.path_result is not None:
            self.path_result = self.simu.path_result.replace("\\", "/")
        else:
            # If the path doesn't exist, set the default one
            self.path_result = abspath(join(RESULT_DIR, self.simu.name))
        if not isdir(self.path_result):
            makedirs(self.path_result)  # Make sure that the folder exist
    return self.path_result



# -*- coding: utf-8 -*-

from .....Functions.Plot.plot_A_quiver_2D import (
    plot_A_quiver_2D as plot_A_quiver_2D_fct,
)


def plot_A_quiver_2D(
    self,
    Data_str,
    t=None,
    t_index=0,
    is_norm=False,
    unit="SI",
    is_show_fig=None,
    save_path=None,
    fig=None,
    ax=None,
):
    """Plots a field as a function of space (angle)

    Parameters
    ----------
    self : Output
        an Output object
    Data_str : str
        name of the Data object to plot (e.g. "mag.Br")
    t : float
        time value at which to slice
    t_index : int
        time index at which to slice
    is_norm : bool
        boolean indicating if the field must be normalized
    unit : str
        unit in which to plot the field
    is_show_fig : bool
        True to show figure after plot
    save_path : str
        full path including folder, name and extension of the file to save if save_path is not None
    fig : Matplotlib.figure.Figure
        existing figure to use if None create a new one
    ax : Matplotlib.axes.Axes object
        ax on which to plot the data
    """

    # Get Data object names
    phys = getattr(self, Data_str.split(".")[0])
    data = getattr(phys, Data_str.split(".")[1])

    # Call the plot function
    plot_A_quiver_2D_fct(
        data,
        t=t,
        t_index=t_index,
        is_norm=is_norm,
        unit=unit,
        is_show_fig=is_show_fig,
        save_path=save_path,
        fig=fig,
        ax=ax,
    )



# -*- coding: utf-8 -*-

from matplotlib.patches import Circle, Patch
from matplotlib.pyplot import axis, legend

from ....Functions.init_fig import init_fig
from ....definitions import config_dict


PATCH_COLOR = config_dict["PLOT"]["COLOR_DICT"]["PATCH_COLOR"]
MAGNET_COLOR = config_dict["PLOT"]["COLOR_DICT"]["MAGNET_COLOR"]
ROTOR_COLOR = config_dict["PLOT"]["COLOR_DICT"]["ROTOR_COLOR"]
STATOR_COLOR = config_dict["PLOT"]["COLOR_DICT"]["STATOR_COLOR"]


def plot(
    self,
    fig=None,
    ax=None,
    is_lam_only=False,
    sym=1,
    alpha=0,
    delta=0,
    is_edge_only=False,
    is_show_fig=True,
):
    """Plot a Lamination with Buried Magnets in a matplotlib fig

    Parameters
    ----------
    self : LamHole
        A LamHole object
    fig : Matplotlib.figure.Figure
        existing figure to use if None create a new one
    ax : Matplotlib.axes.Axes object
        Axis on which to plot the data
    is_lam_only: bool
        True to plot only the lamination (remove the magnets)
    sym : int
        Symmetry factor (1= plot full machine, 2= half of the machine...)
    alpha : float
        angle for rotation (Default value = 0) [rad]
    delta : complex
        complex for translation (Default value = 0)
    is_edge_only: bool
        To plot transparent Patches
    is_show_fig : bool
        To call show at the end of the method

    Returns
    -------
    None
    """

    # Lamination bore
    if self.is_stator:
        lam_color = STATOR_COLOR
    else:
        lam_color = ROTOR_COLOR

    # List of surface to plot the lamination
    surf_list = self.build_geometry(sym=sym, alpha=alpha, delta=delta)
    patches = list()
    for surf in surf_list:
        if surf.label is not None and "Lamination" in surf.label:
            patches.extend(surf.get_patches(color=lam_color, is_edge_only=is_edge_only))
        elif surf.label is not None and "Magnet" in surf.label and not is_lam_only:
            patches.extend(
                surf.get_patches(color=MAGNET_COLOR, is_edge_only=is_edge_only)
            )
        else:
            patches.extend(surf.get_patches(is_edge_only=is_edge_only))

    (fig, axes, patch_leg, label_leg) = init_fig(fig=fig, ax=ax, shape="rectangle")

    axes.set_xlabel("(m)")
    axes.set_ylabel("(m)")

    for patch in patches:
        axes.add_patch(patch)
    # Axis Setup
    axes.axis("equal")

    # The Lamination is centered in the figure
    Lim = self.Rext * 1.5
    axes.set_xlim(-Lim, Lim)
    axes.set_ylim(-Lim, Lim)

    # Set legend
    if not is_edge_only:
        if self.is_stator:
            patch_leg.append(Patch(color=STATOR_COLOR))
            label_leg.append("Stator")
            axes.set_title("Stator with Interior Magnet")
        else:
            patch_leg.append(Patch(color=ROTOR_COLOR))
            label_leg.append("Rotor")
            axes.set_title("Rotor with Interior Magnet")
        if not is_lam_only:
            patch_leg.append(Patch(color=MAGNET_COLOR))
            label_leg.append("Magnet")
        legend(patch_leg, label_leg)
    if is_show_fig:
        fig.show()



# -*- coding: utf-8 -*-
from numpy import pi, angle, exp

from ....Classes.Circle import Circle
from ....Classes.SurfLine import SurfLine
from ....Classes.Arc1 import Arc1
from ....Classes.Segment import Segment
from ....Classes.SurfRing import SurfRing
from ....Classes.Lamination import Lamination


def build_geometry(self, sym=1, alpha=0, delta=0):
    """Build the geometry of the LamSlot object

    Parameters
    ----------
    self : LamSlot
        a LamSlot object
    sym : int
        Symmetry factor (1= full machine, 2= half of the machine...)
    alpha : float
        Angle for rotation [rad]
    delta : complex
        Complex value for translation

    Returns
    -------
    surf_list : list
        list of surfaces needed to draw the lamination

    """

    if self.is_stator:
        label = "Lamination_Stator"  # Label lamination
    else:
        label = "Lamination_Rotor"
    if self.is_internal:
        lb = "Ext"  # label for the bore
        ly = "Int"  # label for the yoke
    else:
        lb = "Int"
        ly = "Ext"
    label_bore = label + "_Bore_Radius_" + lb
    label_yoke = label + "_Yoke_Radius_" + ly

    Ryoke = self.get_Ryoke()
    H_yoke = self.comp_height_yoke()

    if self.slot is not None and self.slot.Zs != 0:
        # getting Number of Slot
        Zs = self.slot.Zs

        # Check for symmetry
        assert (Zs % sym) == 0, (
            "ERROR, Wrong symmetry for "
            + label
            + " "
            + str(Zs)
            + " slots and sym="
            + str(sym)
        )

        bore_desc = self.get_bore_desc(sym=sym)
        bore_list = list()
        for bore in bore_desc:
            if type(bore["obj"]) is Arc1:
                bore_list.append(bore["obj"])
            elif "lines" in bore:  # Duplicated slot
                for line in bore["lines"]:
                    bore_list.append(type(line)(init_dict=line.as_dict()))
                    bore_list[-1].rotate((bore["begin_angle"] + bore["end_angle"]) / 2)
            else:  # Notches
                lines = bore["obj"].build_geometry()
                for line in lines:
                    line.rotate((bore["begin_angle"] + bore["end_angle"]) / 2)
                bore_list.extend(lines)
        # Set line label
        for line in bore_list:
            line.label = label_bore
    else:  # No slot
        return Lamination.build_geometry(self, sym=sym, alpha=alpha, delta=delta)

    # Create the lamination surface(s)
    surf_list = list()
    if sym == 1:  # Complete lamination
        if self.is_internal:
            point_ref = Ryoke + (H_yoke / 2)
        else:
            point_ref = Ryoke - (H_yoke / 2)
        # Create Slot surface
        surf_slot = SurfLine(line_list=bore_list, label=label_bore, point_ref=point_ref)
        # Create yoke circle surface
        surf_yoke = Circle(
            radius=Ryoke, label=label_yoke, line_label=label_yoke, center=0
        )
        if self.Rint == 0 and len(bore_list) > 0:
            surf_list = [surf_slot]
        elif self.Rint == 0 and len(bore_list) == 0:
            surf_list = list()
        elif self.is_internal:
            surf_list.append(
                SurfRing(
                    out_surf=surf_slot,
                    in_surf=surf_yoke,
                    label=label,
                    point_ref=point_ref,
                )
            )
        else:
            surf_list.append(
                SurfRing(
                    out_surf=surf_yoke,
                    in_surf=surf_slot,
                    label=label,
                    point_ref=point_ref,
                )
            )
    else:  # Only one surface
        # Add the Yoke part
        Zy1 = Ryoke
        Zy2 = Ryoke * exp(1j * 2 * pi / sym)
        bore_list.append(
            Segment(bore_list[-1].get_end(), Zy2, label=label + "_Yoke_Side_Left")
        )
        if Ryoke > 0:  # For internal lamination
            bore_list.append(
                Arc1(
                    begin=Zy2,
                    end=Zy1,
                    radius=-Ryoke,
                    is_trigo_direction=False,
                    label=label_yoke,
                )
            )
        bore_list.append(
            Segment(Zy1, bore_list[0].get_begin(), label=label + "_Yoke_Side_Right")
        )
        # Create a Surface for the slot
        if self.is_internal:
            point_ref = (Ryoke + H_yoke / 2) * exp(1j * pi / sym)
        else:
            point_ref = (Ryoke - H_yoke / 2) * exp(1j * pi / sym)
        surf_slot = SurfLine(line_list=bore_list, label=label_bore, point_ref=point_ref)
        surf_list.append(surf_slot)

    # Apply the transformations
    for surf in surf_list:
        surf.rotate(alpha)
        surf.translate(delta)

    # Adding the ventilation surfaces
    for vent in self.axial_vent:
        surf_list += vent.build_geometry(
            sym=sym, alpha=alpha, delta=delta, is_stator=self.is_stator
        )
    return surf_list



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Simulation/StructElmer.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Simulation/StructElmer
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .Structural import Structural

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Simulation.StructElmer.run import run
except ImportError as error:
    run = error

try:
    from ..Methods.Simulation.StructElmer.get_meshsolution import get_meshsolution
except ImportError as error:
    get_meshsolution = error

try:
    from ..Methods.Simulation.StructElmer.get_path_save_fea import get_path_save_fea
except ImportError as error:
    get_path_save_fea = error

try:
    from ..Methods.Simulation.StructElmer.solve_FEA import solve_FEA
except ImportError as error:
    solve_FEA = error

try:
    from ..Methods.Simulation.StructElmer.gen_mesh import gen_mesh
except ImportError as error:
    gen_mesh = error

try:
    from ..Methods.Simulation.StructElmer.gen_case import gen_case
except ImportError as error:
    gen_case = error

try:
    from ..Methods.Simulation.StructElmer.process_mesh import process_mesh
except ImportError as error:
    process_mesh = error


from ._check import InitUnKnowClassError


class StructElmer(Structural):
    """Structural module: FEA model with Elmer"""

    VERSION = 1

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Simulation.StructElmer.run
    if isinstance(run, ImportError):
        run = property(
            fget=lambda x: raise_(
                ImportError("Can't use StructElmer method run: " + str(run))
            )
        )
    else:
        run = run
    # cf Methods.Simulation.StructElmer.get_meshsolution
    if isinstance(get_meshsolution, ImportError):
        get_meshsolution = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use StructElmer method get_meshsolution: "
                    + str(get_meshsolution)
                )
            )
        )
    else:
        get_meshsolution = get_meshsolution
    # cf Methods.Simulation.StructElmer.get_path_save_fea
    if isinstance(get_path_save_fea, ImportError):
        get_path_save_fea = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use StructElmer method get_path_save_fea: "
                    + str(get_path_save_fea)
                )
            )
        )
    else:
        get_path_save_fea = get_path_save_fea
    # cf Methods.Simulation.StructElmer.solve_FEA
    if isinstance(solve_FEA, ImportError):
        solve_FEA = property(
            fget=lambda x: raise_(
                ImportError("Can't use StructElmer method solve_FEA: " + str(solve_FEA))
            )
        )
    else:
        solve_FEA = solve_FEA
    # cf Methods.Simulation.StructElmer.gen_mesh
    if isinstance(gen_mesh, ImportError):
        gen_mesh = property(
            fget=lambda x: raise_(
                ImportError("Can't use StructElmer method gen_mesh: " + str(gen_mesh))
            )
        )
    else:
        gen_mesh = gen_mesh
    # cf Methods.Simulation.StructElmer.gen_case
    if isinstance(gen_case, ImportError):
        gen_case = property(
            fget=lambda x: raise_(
                ImportError("Can't use StructElmer method gen_case: " + str(gen_case))
            )
        )
    else:
        gen_case = gen_case
    # cf Methods.Simulation.StructElmer.process_mesh
    if isinstance(process_mesh, ImportError):
        process_mesh = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use StructElmer method process_mesh: " + str(process_mesh)
                )
            )
        )
    else:
        process_mesh = process_mesh
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        Kmesh_fineness=1,
        path_name="",
        FEA_dict_enforced=-1,
        is_get_mesh=False,
        is_save_FEA=True,
        transform_list=-1,
        include_magnets=True,
        logger_name="Pyleecan.Structural",
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "Kmesh_fineness" in list(init_dict.keys()):
                Kmesh_fineness = init_dict["Kmesh_fineness"]
            if "path_name" in list(init_dict.keys()):
                path_name = init_dict["path_name"]
            if "FEA_dict_enforced" in list(init_dict.keys()):
                FEA_dict_enforced = init_dict["FEA_dict_enforced"]
            if "is_get_mesh" in list(init_dict.keys()):
                is_get_mesh = init_dict["is_get_mesh"]
            if "is_save_FEA" in list(init_dict.keys()):
                is_save_FEA = init_dict["is_save_FEA"]
            if "transform_list" in list(init_dict.keys()):
                transform_list = init_dict["transform_list"]
            if "include_magnets" in list(init_dict.keys()):
                include_magnets = init_dict["include_magnets"]
            if "logger_name" in list(init_dict.keys()):
                logger_name = init_dict["logger_name"]
        # Set the properties (value check and convertion are done in setter)
        self.Kmesh_fineness = Kmesh_fineness
        self.path_name = path_name
        self.FEA_dict_enforced = FEA_dict_enforced
        self.is_get_mesh = is_get_mesh
        self.is_save_FEA = is_save_FEA
        self.transform_list = transform_list
        self.include_magnets = include_magnets
        # Call Structural init
        super(StructElmer, self).__init__(logger_name=logger_name)
        # The class is frozen (in Structural init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        StructElmer_str = ""
        # Get the properties inherited from Structural
        StructElmer_str += super(StructElmer, self).__str__()
        StructElmer_str += "Kmesh_fineness = " + str(self.Kmesh_fineness) + linesep
        StructElmer_str += 'path_name = "' + str(self.path_name) + '"' + linesep
        StructElmer_str += (
            "FEA_dict_enforced = " + str(self.FEA_dict_enforced) + linesep
        )
        StructElmer_str += "is_get_mesh = " + str(self.is_get_mesh) + linesep
        StructElmer_str += "is_save_FEA = " + str(self.is_save_FEA) + linesep
        StructElmer_str += (
            "transform_list = "
            + linesep
            + str(self.transform_list).replace(linesep, linesep + "\t")
            + linesep
        )
        StructElmer_str += "include_magnets = " + str(self.include_magnets) + linesep
        return StructElmer_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from Structural
        if not super(StructElmer, self).__eq__(other):
            return False
        if other.Kmesh_fineness != self.Kmesh_fineness:
            return False
        if other.path_name != self.path_name:
            return False
        if other.FEA_dict_enforced != self.FEA_dict_enforced:
            return False
        if other.is_get_mesh != self.is_get_mesh:
            return False
        if other.is_save_FEA != self.is_save_FEA:
            return False
        if other.transform_list != self.transform_list:
            return False
        if other.include_magnets != self.include_magnets:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from Structural
        diff_list.extend(super(StructElmer, self).compare(other, name=name))
        if other._Kmesh_fineness != self._Kmesh_fineness:
            diff_list.append(name + ".Kmesh_fineness")
        if other._path_name != self._path_name:
            diff_list.append(name + ".path_name")
        if other._FEA_dict_enforced != self._FEA_dict_enforced:
            diff_list.append(name + ".FEA_dict_enforced")
        if other._is_get_mesh != self._is_get_mesh:
            diff_list.append(name + ".is_get_mesh")
        if other._is_save_FEA != self._is_save_FEA:
            diff_list.append(name + ".is_save_FEA")
        if other._transform_list != self._transform_list:
            diff_list.append(name + ".transform_list")
        if other._include_magnets != self._include_magnets:
            diff_list.append(name + ".include_magnets")
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from Structural
        S += super(StructElmer, self).__sizeof__()
        S += getsizeof(self.Kmesh_fineness)
        S += getsizeof(self.path_name)
        if self.FEA_dict_enforced is not None:
            for key, value in self.FEA_dict_enforced.items():
                S += getsizeof(value) + getsizeof(key)
        S += getsizeof(self.is_get_mesh)
        S += getsizeof(self.is_save_FEA)
        if self.transform_list is not None:
            for value in self.transform_list:
                S += getsizeof(value)
        S += getsizeof(self.include_magnets)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        # Get the properties inherited from Structural
        StructElmer_dict = super(StructElmer, self).as_dict(**kwargs)
        StructElmer_dict["Kmesh_fineness"] = self.Kmesh_fineness
        StructElmer_dict["path_name"] = self.path_name
        StructElmer_dict["FEA_dict_enforced"] = (
            self.FEA_dict_enforced.copy()
            if self.FEA_dict_enforced is not None
            else None
        )
        StructElmer_dict["is_get_mesh"] = self.is_get_mesh
        StructElmer_dict["is_save_FEA"] = self.is_save_FEA
        StructElmer_dict["transform_list"] = (
            self.transform_list.copy() if self.transform_list is not None else None
        )
        StructElmer_dict["include_magnets"] = self.include_magnets
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        StructElmer_dict["__class__"] = "StructElmer"
        return StructElmer_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.Kmesh_fineness = None
        self.path_name = None
        self.FEA_dict_enforced = None
        self.is_get_mesh = None
        self.is_save_FEA = None
        self.transform_list = None
        self.include_magnets = None
        # Set to None the properties inherited from Structural
        super(StructElmer, self)._set_None()

    def _get_Kmesh_fineness(self):
        """getter of Kmesh_fineness"""
        return self._Kmesh_fineness

    def _set_Kmesh_fineness(self, value):
        """setter of Kmesh_fineness"""
        check_var("Kmesh_fineness", value, "float")
        self._Kmesh_fineness = value

    Kmesh_fineness = property(
        fget=_get_Kmesh_fineness,
        fset=_set_Kmesh_fineness,
        doc=u"""global coefficient to adjust mesh fineness in FEMM (1 : default , > 1 : finner , < 1 : less fine)

        :Type: float
        """,
    )

    def _get_path_name(self):
        """getter of path_name"""
        return self._path_name

    def _set_path_name(self, value):
        """setter of path_name"""
        check_var("path_name", value, "str")
        self._path_name = value

    path_name = property(
        fget=_get_path_name,
        fset=_set_path_name,
        doc=u"""Name of the path to save the FEA model

        :Type: str
        """,
    )

    def _get_FEA_dict_enforced(self):
        """getter of FEA_dict_enforced"""
        return self._FEA_dict_enforced

    def _set_FEA_dict_enforced(self, value):
        """setter of FEA_dict_enforced"""
        if type(value) is int and value == -1:
            value = dict()
        check_var("FEA_dict_enforced", value, "dict")
        self._FEA_dict_enforced = value

    FEA_dict_enforced = property(
        fget=_get_FEA_dict_enforced,
        fset=_set_FEA_dict_enforced,
        doc=u"""To enforce user-defined values for FEA main parameters 

        :Type: dict
        """,
    )

    def _get_is_get_mesh(self):
        """getter of is_get_mesh"""
        return self._is_get_mesh

    def _set_is_get_mesh(self, value):
        """setter of is_get_mesh"""
        check_var("is_get_mesh", value, "bool")
        self._is_get_mesh = value

    is_get_mesh = property(
        fget=_get_is_get_mesh,
        fset=_set_is_get_mesh,
        doc=u"""To save FEA mesh for latter post-procesing (only possible with is_save_FEA set to True)

        :Type: bool
        """,
    )

    def _get_is_save_FEA(self):
        """getter of is_save_FEA"""
        return self._is_save_FEA

    def _set_is_save_FEA(self, value):
        """setter of is_save_FEA"""
        check_var("is_save_FEA", value, "bool")
        self._is_save_FEA = value

    is_save_FEA = property(
        fget=_get_is_save_FEA,
        fset=_set_is_save_FEA,
        doc=u"""To save FEA mesh and solution in .vtu file

        :Type: bool
        """,
    )

    def _get_transform_list(self):
        """getter of transform_list"""
        return self._transform_list

    def _set_transform_list(self, value):
        """setter of transform_list"""
        if type(value) is int and value == -1:
            value = list()
        check_var("transform_list", value, "list")
        self._transform_list = value

    transform_list = property(
        fget=_get_transform_list,
        fset=_set_transform_list,
        doc=u"""List of dictionnary to apply transformation on the machine surfaces. Key: label (to select the surface), type (rotate or translate), value (alpha or delta)

        :Type: list
        """,
    )

    def _get_include_magnets(self):
        """getter of include_magnets"""
        return self._include_magnets

    def _set_include_magnets(self, value):
        """setter of include_magnets"""
        check_var("include_magnets", value, "bool")
        self._include_magnets = value

    include_magnets = property(
        fget=_get_include_magnets,
        fset=_set_include_magnets,
        doc=u"""Switch to include magents in the structural simulation

        :Type: bool
        """,
    )



try:
    import setuptools
except ImportError:  # Install setuptools if needed
    from os import system
    from sys import executable

    # run 'pip install setuptools'
    system("{} -m pip install setuptools".format(executable))

    import setuptools

import platform


# /!\ Increase the number before a release
# See https://www.python.org/dev/peps/pep-0440/
# Examples :
# First alpha of the release 0.1.0 : 0.1.0a1
# First beta of the release 1.0.0 : 1.0.0b1
# Second release candidate of the release 2.6.4 : 2.6.4rc2
# Release 1.1.0 : 1.1.0
# First post release of the release 1.1.0 : 1.1.0.post1

PYLEECAN_VERSION = "1.2.0"


with open("README.md", "r") as fh:
    long_description = fh.read()

python_requires = ">= 3.6"

# Pyleecan dependancies
install_requires = [
    "setuptools",
    "cloudpickle>=1.3.0",
    "matplotlib>=3.3.2",
    "numpy==1.19.3",
    "pandas>=1.0.3",
    'pyfemm>=0.1.1;platform_system=="Windows"',
    "PySide2>=5.15.1",
    "scipy>=1.4.1",
    "xlrd>=1.2.0",
    "deap>=1.3.1",
    "SciDataTool>=1.3.6",
    "pyvista>=0.25.3",
    "meshio>=4.0.15",
    "h5py==2.10.0",
    "nbformat",
    "nbconvert",
    "quadpy",
    "ezdxf>=0.14.2",
    "pytest-qt>=3.3.0",
]


tests_require = ["ddt>=1.3.1", "pytest>=5.4.1", "mock>=4.0.2", "nbformat", "nbconvert"]

setuptools.setup(
    name="pyleecan",
    version=PYLEECAN_VERSION,
    author="Pyleecan Developers",
    author_email="pyleecan@framalistes.org",
    description="Python Library for Electrical Engineering Computational Analysis",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Eomys/pyleecan",
    download_url="https://github.com/Eomys/pyleecan/archive/"
    + PYLEECAN_VERSION
    + ".tar.gz",
    packages=setuptools.find_packages(exclude=["Tests*", "Tutorials*"]),
    package_data={
        # Include any *.json files found in pyleecan:
        # '': ['*.json'],
        "pyleecan": ["*.json"]
    },
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    python_requires=python_requires,
    install_requires=install_requires,
    extras_require={
        "test": tests_require
    },  # Enables to install the test dependancies using pip install pyleecan[test]
)



# -*- coding: utf-8 -*-

import numpy as np


def grad_shape_function(self, point):
    """ Return the gradient of linear shape functions in reference triangle for a given point """
    # TODO
    return None



from PySide2.QtCore import Qt, Signal
from PySide2.QtWidgets import QWidget

from ....Classes.ImportGenVectLin import ImportGenVectLin
from ....GUI.Tools.WImport.Ui_WImport import Ui_WImport
from ....GUI.Tools.WImport.WImportExcel.WImportExcel import WImportExcel
from ....GUI.Tools.WImport.WImportLinspace.WImportLinspace import WImportLinspace
from ....GUI.Tools.WImport.WImportMatrixTable.WImportMatrixTable import (
    WImportMatrixTable,
)


class WImport(Ui_WImport, QWidget):
    saveNeeded = Signal()

    def __init__(self, parent=None):
        """Widget to define an ImportLinspace"""
        QWidget.__init__(self, parent=parent)
        self.setupUi(self)

        self.obj = None  # Object to edit
        self.verbose_name = ""  # Name to display / adapt the GUI
        # List to enforce a shape, [None, 2] enforce 2D matrix with 2 columns
        self.expected_shape = None
        self.param_name = ""  # Name of the quantity to set
        self.widget_list = []  # Available widget to import

        self.c_type_import.currentIndexChanged.connect(self.set_type_import)

    def update(self):
        """Update the display of the Widget"""
        self.in_param.setText(self.verbose_name)
        # Fill the combobox with the meaningful widgets
        self.c_type_import.blockSignals(True)
        if self.verbose_name == "B(H) curve":
            self.widget_list = [WImportExcel, WImportMatrixTable]
        else:
            self.widget_list = []
        # Generate the Combobox content
        self.c_type_import.clear()
        for widget in self.widget_list:
            self.c_type_import.addItem(widget.import_name)
        # Set the index according to data type
        type_list = [wid.import_type for wid in self.widget_list]
        data = getattr(self.obj, self.param_name)
        if type(data) not in type_list:
            # Initialization with first time
            data = type_list[0]()
            data._set_None()
            setattr(self.obj, self.param_name, data)
            self.emit_save()
        self.c_type_import.setCurrentIndex(type_list.index(type(data)))
        self.set_import_widget()
        self.c_type_import.blockSignals(False)

    def set_type_import(self):
        """Change the type of the import object according to the combobox"""
        data = self.widget_list[self.c_type_import.currentIndex()].import_type()
        data._set_None()
        setattr(self.obj, self.param_name, data)
        self.emit_save()
        self.set_import_widget()

    def set_import_widget(self):
        """Update the import widget

        Parameters
        ----------
        self : WImport
            A WImport object
        """
        data = getattr(self.obj, self.param_name)
        # Regenerate the pages with the new values
        self.w_import.setParent(None)
        self.w_import = self.widget_list[self.c_type_import.currentIndex()](
            data=data,
            verbose_name=self.verbose_name,
            expected_shape=self.expected_shape,
        )
        self.w_import.data_type = self.verbose_name
        self.w_import.saveNeeded.connect(self.emit_save)
        self.w_import.dataTypeChanged.connect(self.update_type)
        # Refresh the GUI
        self.main_layout.removeWidget(self.w_import)
        self.main_layout.insertWidget(1, self.w_import)

    def update_type(self):
        """The Import widget has changed the type of the import"""
        setattr(self.obj, self.param_name, self.w_import.data)
        self.update()

    def emit_save(self):
        """Send a saveNeeded signal to the DMachineSetup"""
        self.saveNeeded.emit()
